@@ -12,13 +12,12 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <QtCore/QString>
-#include <QtCore/QFile>
+#include <QString>
+#include <QFile>
 
 #define XCA_MAGIC 0xcadb1969
 #define NAMELEN 80
 #define FNAMLEN 256
-#define OFF_EOF ((off_t)-1)
 
 #define DBFLAG_DELETED  0x1
 #define DBFLAG_OUTDATED 0x2
@@ -28,7 +27,7 @@ enum pki_type {
 	asym_key,
 	x509_req,
 	x509,
-	revokation,
+	revocation,
 	tmpl,
 	setting,
 	smartCard,
@@ -57,10 +56,12 @@ class db
 		QString name);
 	void convert_header(db_header_t *h);
 	void fileIOerr(QString s);
+	QString backup_name();
+	bool backup();
 
     public:
 	bool verify_magic(void);
-	off_t head_offset;
+	qint64 head_offset;
 	db(QString, QFlags<QFile::Permission> perm = QFile::ReadOwner | QFile::WriteOwner);
 	~db();
 	bool eof();
