@@ -1,18 +1,24 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2014 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2015 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
+#ifdef WIN32
+#include <windows.h>
+#else
+#include <netinet/in.h>
+#endif
 
 #include "db_base.h"
 #include "func.h"
 #include "exception.h"
-#include <QtGui/QMessageBox>
-#include <QtGui/QListView>
-#include <QtGui/QClipboard>
-#include <QtCore/QDir>
+#include <QMessageBox>
+#include <QListView>
+#include <QClipboard>
+#include <QDir>
+#include <QDebug>
 #include "widgets/MainWindow.h"
 #include "widgets/ImportMulti.h"
 
@@ -25,7 +31,6 @@ db_base::db_base(QString db, MainWindow *mw)
 	colResizing = 0;
 	currentIdx = QModelIndex();
 	class_name = "base";
-	allHeaders << new dbheader(HD_internal_name, true, tr("Internal name"));
 }
 
 db_base::~db_base()
@@ -64,6 +69,55 @@ void db_base::remFromCont(QModelIndex &idx)
 	emit columnsContentChanged();
 }
 
+int db_base::handleBadEntry(unsigned char *p, db_header_t *head)
+{
+	QString name = QString::fromUtf8(head->name);
+	QString txt = tr("Bad database item\nName: %1\nType: %2\nSize: %3\n%4")
+		.arg(name).arg(class_name)
+		.arg(head->len - sizeof(db_header_t))
+		.arg(tr("Do you want to delete the item from the database? The bad item may be extracted into a separate file."));
+
+	xcaWarning msg(mainwin, txt);
+	msg.addButton(QMessageBox::Ok)->setText(tr("Delete"));
+	msg.addButton(QMessageBox::Apply)->setText(tr("Delete and extract"));
+	msg.addButton(QMessageBox::Cancel)->setText(tr("Continue"));
+
+	switch (msg.exec()) {
+	case QMessageBox::Ok:
+		return 1;
+	case QMessageBox::Cancel:
+	default:
+		return 0;
+	case QMessageBox::Apply:
+		break;
+	}
+
+	QString s = QFileDialog::getSaveFileName(mainwin, QString(), name,
+			QString(), NULL, QFileDialog::DontConfirmOverwrite);
+
+	size_t l;
+	db_header_t h;
+	FILE *fp = fopen_write(s);
+	if (!fp) {
+		throw errorEx(tr("Error opening file: '%1': %2").
+                        arg(s).arg(strerror(errno)), class_name);
+	}
+
+	h.magic   = ntohl(head->magic);
+	h.len     = ntohl(head->len);
+	h.headver = ntohs(head->headver);
+	h.type    = ntohs(head->type);
+	h.version = ntohs(head->version);
+	h.flags   = ntohs(head->flags);
+	memcpy(h.name, head->name, NAMELEN);
+
+	l = fwrite(&h, sizeof h, 1, fp);
+	l += fwrite(p, head->len - sizeof h, 1, fp);
+	fclose(fp);
+
+	return (l == 2);
+}
+
 void db_base::loadContainer()
 {
 	db mydb(dbName);
@@ -77,13 +131,13 @@ void db_base::loadContainer()
 			QString s;
 			p = mydb.load(&head);
 			if (!p) {
-				printf("Load was empty !\n");
+				qWarning("Load was empty !");
 				goto next;
 			}
 			pki = newPKI(&head);
 			if (pki->getVersion() < head.version) {
-				printf("Item[%s]: Version %d "
-					"> known version: %d -> ignored\n",
+				qWarning("Item[%s]: Version %d "
+					"> known version: %d -> ignored",
 					head.name, head.version,
 					pki->getVersion()
 				);
@@ -101,6 +155,13 @@ void db_base::loadContainer()
 				mainwin->Error(err);
 				delete pki;
 				pki = NULL;
+				try {
+					if (handleBadEntry(p, &head)) {
+						mydb.erase();
+					}
+				} catch (errorEx &err) {
+					mainwin->Error(err);
+				}
 			}
 			free(p);
 			if (pki) {
@@ -134,12 +195,30 @@ next:
 	return;
 }
 
+void db_base::updateHeaders()
+{
+	QByteArray ba = allHeaders.toData();
+	foreach(dbheader *h, allHeaders)
+		delete h;
+	allHeaders = getHeaders();
+	allHeaders.fromData(ba);
+}
+
+dbheaderList db_base::getHeaders()
+{
+	dbheaderList h;
+	/* "No." handled in XcaProxyModel */
+	h << new dbheader(HD_internal_name, true, tr("Internal name"))
+	  << new dbheader(HD_counter, false, tr("No."));
+	return h;
+}
+
+
 void db_base::saveHeaderState()
 {
-	QByteArray ba;
 	if (dbName.isEmpty())
 		return;
-	ba = allHeaders.toData();
+	QByteArray ba = allHeaders.toData();
 	db mydb(dbName);
 	mydb.set((const unsigned char *)ba.constData(), ba.size(), 5,
 		setting, class_name + "_hdView");
@@ -207,45 +286,41 @@ void db_base::insertPKI(pki_base *pki)
 	emit columnsContentChanged();
 }
 
-void db_base::delete_ask()
+QString db_base::pem2QString(QModelIndexList indexes)
 {
-	if (!currentIdx.isValid())
-		return;
-	pki_base *pki = static_cast<pki_base*>(currentIdx.internalPointer());
+	exportType::etype format;
+	QString msg;
 
-	if (!XCA_OKCANCEL(pki->getMsg(pki_base::msg_delete).arg(pki->getIntName())))
-		 return;
-
-	deletePKI();
+	format = clipboardFormat(indexes);
+	foreach(QModelIndex idx, indexes) {
+		long l;
+		const char *p;
+		BIO *bio = BIO_new(BIO_s_mem());
+		pki_base *pki = static_cast<pki_base*>
+				(idx.internalPointer());
+		pki->pem(bio, format);
+		openssl_error();
+		l = BIO_get_mem_data(bio, &p);
+		msg += QString::fromUtf8(p, l);
+		BIO_free(bio);
+	}
+	return msg;
 }
 
-void db_base::pem2clipboard()
+void db_base::pem2clipboard(QModelIndexList indexes)
 {
-	long l;
-        const char *p;
-	QString msg;
+	QString msg = pem2QString(indexes);
 	QClipboard *cb = QApplication::clipboard();
 
-	BIO *bio = BIO_new(BIO_s_mem());
-	if (!currentIdx.isValid())
-		return;
-	pki_base *pki = static_cast<pki_base*>(currentIdx.internalPointer());
-	pki->pem(bio);
-	openssl_error();
-	l = BIO_get_mem_data(bio, &p);
-	msg = QString::fromUtf8(p, l);
-	BIO_free(bio);
 	if (cb->supportsSelection())
 		cb->setText(msg, QClipboard::Selection);
 	else
 		cb->setText(msg);
 }
 
-void db_base::deletePKI()
+void db_base::deletePKI(QModelIndex idx)
 {
-	if (!currentIdx.isValid())
-		return;
-	pki_base *pki = static_cast<pki_base*>(currentIdx.internalPointer());
+	pki_base *pki = static_cast<pki_base*>(idx.internalPointer());
 	try {
 		try {
 			pki->deleteFromToken();
@@ -253,7 +328,7 @@ void db_base::deletePKI()
 			MainWindow::Error(err);
 		}
 
-		remFromCont(currentIdx);
+		remFromCont(idx);
 
 		db mydb(dbName);
 		mydb.find(pki->getType(), pki->getIntName());
@@ -276,80 +351,9 @@ void db_base::updatePKI(pki_base *pki)
 	}
 }
 
-void db_base::deleteSelectedItems(XcaTreeView* view)
-{
-	QModelIndexList indexes = view->getSelectedIndexes();
-	QModelIndex index;
-	QString items, msg;
-	int count = 0;
-	pki_base *pki = NULL;
-
-	if (indexes.count() == 0)
-		return;
-
-	foreach(index, indexes) {
-		if (index.column() != 0)
-			continue;
-		pki = static_cast<pki_base*>(index.internalPointer());
-		items += "'" + pki->getIntName() + "' ";
-		count++;
-	}
-	if (count == 1)
-		msg = pki->getMsg(pki_base::msg_delete).arg(pki->getIntName());
-	else
-		msg = pki->getMsg(pki_base::msg_delete_multi).arg(count).
-				arg(items);
-
-	if (!XCA_OKCANCEL(msg))
-		return;
-
-	foreach(index, indexes) {
-		if (index.column() != 0)
-			continue;
-		currentIdx = index;
-		deletePKI();
-	}
-	currentIdx = QModelIndex();
-}
-
-void db_base::showSelectedItems(XcaTreeView* view)
-{
-	QModelIndexList indexes = view->getSelectedIndexes();
-	QModelIndex index;
-	QString items;
-
-	foreach(index, indexes) {
-		if (index.column() != 0)
-			continue;
-		currentIdx = index;
-		showItem();
-	}
-	currentIdx = QModelIndex();
-}
-
-void db_base::storeSelectedItems(XcaTreeView* view)
-{
-	QModelIndexList indexes = view->getSelectedIndexes();
-	QModelIndex index;
-	QString items;
-
-	foreach(index, indexes) {
-		if (index.column() != 0)
-			continue;
-		try {
-			currentIdx = index;
-			store();
-		} catch (errorEx &err) {
-			MainWindow::Error(err);
-		}
-
-	}
-	currentIdx = QModelIndex();
-}
-
 void db_base::showItem(const QModelIndex &index)
 {
-	showPki(static_cast<pki_key*>(index.internalPointer()));
+	showPki(static_cast<pki_base*>(index.internalPointer()));
 }
 
 void db_base::showItem(const QString name)
@@ -456,6 +460,8 @@ QModelIndex db_base::index(int row, int column, const QModelIndex &parent)
 
 QModelIndex db_base::index(pki_base *pki) const
 {
+	if (!pki)
+		return QModelIndex();
 	return createIndex(pki->row(), 0, pki);
 }
 
@@ -466,14 +472,7 @@ QModelIndex db_base::parent(const QModelIndex &idx) const
 
 	pki_base *childItem = static_cast<pki_base*>(idx.internalPointer());
 	pki_base *parentItem = childItem->getParent();
-#if 0
-	if (parentItem == NULL) {
-		printf("Item: (%s) %s: parent == NULL\n",
-			childItem->className(), CCHAR(childItem->getIntName()));
-	}
-	printf("Parent of:%s(%p) %p %p\n",
-	CCHAR(childItem->getIntName()), childItem, parentItem, rootItem);
-#endif
+
 	if (parentItem == rootItem || parentItem == NULL)
 		return QModelIndex();
 
@@ -489,7 +488,6 @@ int db_base::rowCount(const QModelIndex &parent) const
 	else
 		parentItem = static_cast<pki_base*>(parent.internalPointer());
 
-	//printf("%s rows=%d\n", CCHAR(parentItem->getIntName()), parentItem->childCount());
 	return parentItem->childCount();
 }
 
@@ -512,37 +510,37 @@ QVariant db_base::data(const QModelIndex &index, int role) const
 			return item->getIcon(hd);
 		case Qt::TextAlignmentRole:
 			return hd->isNumeric() ? Qt::AlignRight : Qt::AlignLeft;
-		case Qt::FontRole: {
-			if (hd->isNumeric())
-				return QVariant(QFont("Monospace"));
-			return QVariant(QApplication::font());
-		}
-		case Qt::BackgroundRole: {
+		case Qt::FontRole:
+			return QVariant(XCA_application::tableFont);
+		case Qt::BackgroundRole:
 			return item->bg_color(hd);
-		}
+		case Qt::UserRole:
+			return item->visible();
 	}
 	return QVariant();
 }
-#if 0
-static QString getHeaderViewInfo(int sect, dbheader *h)
+static QVariant getHeaderViewInfo(dbheader *h)
 {
-	return QString("H[%1] Show:%2%3 Size:%4 VI:%5 Indi:%6").
+	return QVariant(
+#if 0
+	QString("H[%1] Show:%2%3 Size:%4 VI:%5 Indi:%6").
 		arg(sect).arg(h->show).arg(h->showDefault).arg(h->size).
-		arg(h->visualIndex).arg(h->sortIndicator);
-}
+		arg(h->visualIndex).arg(h->sortIndicator)
+#else
+	h->getTooltip()
 #endif
+	);
+}
+
 QVariant db_base::headerData(int section, Qt::Orientation orientation,
 		int role) const
 {
 	if (orientation == Qt::Horizontal) {
 		switch (role) {
 		case Qt::DisplayRole:
-			return QVariant(allHeaders[section]->name);
+			return QVariant(allHeaders[section]->getName());
 		case Qt::ToolTipRole:
-#if 0
-			return getHeaderViewInfo(section, allHeaders[section]);
-#endif
-			return QVariant(allHeaders[section]->tooltip);
+			return getHeaderViewInfo(allHeaders[section]);
 		}
 	}
 	return QVariant();
@@ -610,22 +608,52 @@ void db_base::load_default(load_base &load)
 	delete dlgi;
 }
 
-void db_base::edit()
+void db_base::store(QModelIndexList indexes)
 {
-	if (!currentIdx.isValid())
-		return;
-	emit editItem(currentIdx);
-}
+	int ret;
 
-void db_base::showItem()
-{
-	if (!currentIdx.isValid())
+	xcaWarning msg(mainwin, tr("How to export the %1 selected items").
+				arg(indexes.size()));
+	msg.addButton(QMessageBox::Ok)->setText(tr("All in one PEM file"));
+	msg.addButton(QMessageBox::Apply)->setText(tr("Each item in one file"));
+	msg.addButton(QMessageBox::Cancel);
+	ret = msg.exec();
+	if (ret == QMessageBox::Apply) {
+		foreach(QModelIndex i, indexes)
+			store(i);
 		return;
-	showItem(currentIdx);
+	} else if (ret != QMessageBox::Ok) {
+		return;
+	}
+
+	QString fn = mainwin->getPath() + QDir::separator() + "export.pem";
+	QString s = QFileDialog::getSaveFileName(mainwin,
+		tr("Save %1 items in one file as").arg(indexes.size()), fn,
+		tr("PEM Files( *.pem );; All files ( * )"));
+	if (s.isEmpty())
+		return;
+	s = nativeSeparator(s);
+	mainwin->setPath(s.mid(0, s.lastIndexOf(QRegExp("[/\\\\]")) ));
+	try {
+		QString pem = pem2QString(indexes);
+		FILE *fp = fopen_write(s);
+		if (!fp) {
+			throw errorEx(tr("Error opening file: '%1': %2").
+				arg(s).arg(strerror(errno)), class_name);
+		}
+		fwrite(pem.toLatin1(), pem.size(), 1, fp);
+		fclose(fp);
+	}
+	catch (errorEx &err) {
+		MainWindow::Error(err);
+	}
 }
 
 bool db_base::columnHidden(int col) const
 {
+	if (pki_x509::disable_netscape &&
+	    allHeaders[col]->type == dbheader::hd_v3ext_ns)
+		return true;
 	return !allHeaders[col]->show;
 }
 
@@ -642,68 +670,3 @@ bool db_base::isNumericCol(int col) const
 {
 	return allHeaders[col]->isNumeric();
 }
-
-void db_base::showHeaderMenu(QContextMenuEvent *e, int sect)
-{
-	contextMenu(e, NULL, sect);
-}
-
-void db_base::contextMenu(QContextMenuEvent *e, QMenu *parent, int)
-{
-	int shown = 0;
-	QMenu *menu, *dn, *v3ext, *current;
-	QAction *a, *sep;
-	dbheader *hd;
-
-	menu = new QMenu(mainwin);
-	dn = new QMenu(tr("Subject entries"));
-	v3ext = new QMenu(tr("X509v3 Extensions"));
-	menu->addAction(tr("Reset"), this, SLOT(columnResetDefaults()));
-	sep = menu->addSeparator();
-	foreach(hd, allHeaders) {
-		switch (hd->type) {
-			case dbheader::hd_x509name:
-				current = dn;
-				break;
-			case dbheader::hd_v3ext:
-				current = v3ext;
-				break;
-			default:
-				current = menu;
-				break;
-		}
-		a = current->addAction(hd->name);
-		a->setCheckable(true);
-		a->setChecked(hd->show);
-		if (!hd->tooltip.isEmpty())
-			a->setToolTip(hd->tooltip);
-		hd->action = a;
-	}
-	if (!dn->isEmpty() || !v3ext->isEmpty())
-		menu->insertSeparator(sep);
-	if (!dn->isEmpty())
-		menu->insertMenu(sep, dn);
-	if (!v3ext->isEmpty())
-		menu->insertMenu(sep, v3ext);
-
-	if (parent) {
-		parent->addAction(tr("Paste PEM data"), mainwin,
-				SLOT(pastePem()));
-		parent->addMenu(menu)->setText(tr("Columns"));
-		parent->exec(e->globalPos());
-	} else {
-		menu->exec(e->globalPos());
-	}
-	foreach(hd, allHeaders) {
-		if (hd->action)
-			hd->show = hd->action->isChecked();
-		shown += hd->show ? 1 : 0;
-		hd->action = NULL;
-	}
-	if (!shown)
-		allHeaders[0]->show = true;
-        delete menu;
-	if (parent)
-		delete parent;
-	emit updateHeader();
-}
