@@ -10,20 +10,20 @@
 #include "func.h"
 #include <widgets/NewX509.h>
 #include <widgets/MainWindow.h>
-#include <QtGui/QFileDialog>
-#include <QtCore/QDir>
-#include <QtGui/QContextMenuEvent>
-#include <QtGui/QAction>
-#include <QtGui/QInputDialog>
-#include <QtGui/QMessageBox>
+#include <QFileDialog>
+#include <QDir>
+#include <QContextMenuEvent>
+#include <QAction>
+#include <QInputDialog>
+#include <QMessageBox>
 
 db_temp::db_temp(QString DBfile, MainWindow *mw)
 	:db_x509name(DBfile, mw)
 {
-	allHeaders << new dbheader(HD_temp_type, true, tr("Type"));
 	class_name = "templates";
 	pkitype << tmpl;
 
+	updateHeaders();
 	loadContainer();
 
 	predefs = newPKI();
@@ -50,6 +50,13 @@ db_temp::db_temp(QString DBfile, MainWindow *mw)
 	}
 }
 
+dbheaderList db_temp::getHeaders()
+{
+	dbheaderList h = db_x509name::getHeaders();
+	h << new dbheader(HD_temp_type, true, tr("Type"));
+	return h;
+}
+
 db_temp::~db_temp()
 {
 	delete predefs;
@@ -113,8 +120,8 @@ void db_temp::newItem()
 	for (i=0; i<len; i++) {
 		sl << predefs->child(i)->getIntName();
 	}
-	type = QInputDialog::getItem(mainwin, tr(XCA_TITLE),
-		tr("Preset Template values"), sl, 0, false, &ok, 0 );
+	type = QInputDialog::getItem(mainwin, XCA_TITLE,
+		tr("Preset Template values"), sl, 0, false, &ok, 0);
 	if (ok) {
 		if (type == sl[0]) {
 			temp = new pki_temp("");
@@ -138,36 +145,9 @@ void db_temp::newItem()
 	}
 	delete temp;
 }
-
-void db_temp::changeTemp()
-{
-	if (!currentIdx.isValid())
-		return;
-	pki_temp *temp = static_cast<pki_temp*>(currentIdx.internalPointer());
-	alterTemp(temp);
-}
-
-void db_temp::duplicateTemp()
-{
-	if (!currentIdx.isValid())
-		return;
-	pki_temp *temp = static_cast<pki_temp*>(currentIdx.internalPointer());
-	pki_temp *newtemp = new pki_temp(temp);
-	newtemp->setIntName(newtemp->getIntName() + " " + tr("copy"));
-	insertPKI(newtemp);
-}
-
-bool db_temp::alterTemp(pki_temp *temp)
-{
-	if (!runTempDlg(temp))
-		return false;
-	updatePKI(temp);
-	return true;
-}
-
 void db_temp::showPki(pki_base *pki)
 {
-	alterTemp((pki_temp *)pki);
+	alterTemp(static_cast<pki_temp *>(pki));
 }
 
 void db_temp::load()
@@ -176,11 +156,12 @@ void db_temp::load()
 	load_default(l);
 }
 
-void db_temp::store()
+void db_temp::store(QModelIndex index)
 {
-	if (!currentIdx.isValid())
+	if (!index.isValid())
 		return;
-	pki_temp *temp = static_cast<pki_temp*>(currentIdx.internalPointer());
+
+	pki_temp *temp = static_cast<pki_temp*>(index.internalPointer());
 
 	QString fn = mainwin->getPath() + QDir::separator() +
 		temp->getUnderlinedName() + ".xca";
@@ -189,7 +170,7 @@ void db_temp::store()
 		tr("XCA templates ( *.xca);; All files ( * )"));
 	if (s.isEmpty())
 		return;
-	s = QDir::convertSeparators(s);
+	s = nativeSeparator(s);
 	mainwin->setPath(s.mid(0, s.lastIndexOf(QRegExp("[/\\\\]")) ));
 	try {
 		temp->writeTemp(s);
@@ -199,48 +180,10 @@ void db_temp::store()
 	}
 }
 
-void db_temp::certFromTemp()
+bool db_temp::alterTemp(pki_temp *temp)
 {
-	if (!currentIdx.isValid())
-		return;
-	pki_temp *temp = static_cast<pki_temp*>(currentIdx.internalPointer());
-	emit newCert(temp);
-}
-
-void db_temp::reqFromTemp()
-{
-	if (!currentIdx.isValid())
-		return;
-	pki_temp *temp = static_cast<pki_temp*>(currentIdx.internalPointer());
-	emit newReq(temp);
-}
-
-void db_temp::alterTemp()
-{
-	if (!currentIdx.isValid())
-		return;
-	pki_temp *temp = static_cast<pki_temp*>(currentIdx.internalPointer());
-	alterTemp(temp);
-}
-
-void db_temp::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
-{
-	QMenu *menu = new QMenu(mainwin);
-	currentIdx = index;
-
-	menu->addAction(tr("New Template"), this, SLOT(newItem()));
-	menu->addAction(tr("Import"), this, SLOT(load()));
-	if (index != QModelIndex()) {
-		menu->addAction(tr("Rename"), this, SLOT(edit()));
-		menu->addAction(tr("Export"), this, SLOT(store()));
-		menu->addAction(tr("Change"), this, SLOT(alterTemp()));
-		menu->addAction(tr("Delete"), this, SLOT(delete_ask()));
-		menu->addAction(tr("Duplicate"), this, SLOT(duplicateTemp()));
-		menu->addAction(tr("Create certificate"), this,
-				SLOT(certFromTemp()));
-		menu->addAction(tr("Create request"), this, SLOT(reqFromTemp()));
-	}
-	contextMenu(e, menu);
-	currentIdx = QModelIndex();
-	return;
+	if (!runTempDlg(temp))
+		return false;
+	updatePKI(temp);
+	return true;
 }
