@@ -12,10 +12,8 @@
 #include "db_base.h"
 #include "pki_temp.h"
 #include "db_x509super.h"
-#include <QtCore/QObject>
-#include <QtGui/QPixmap>
-
-class Mainwin;
+#include <QObject>
+#include <QPixmap>
 
 class db_temp: public db_x509name
 {
@@ -23,29 +21,20 @@ class db_temp: public db_x509name
     protected:
 	QPixmap *keyicon;
 	pki_base *predefs;
+	dbheaderList getHeaders();
+
     public:
 	db_temp(QString DBfile, MainWindow *mw);
 	~db_temp();
 	pki_base *newPKI(db_header_t *head = NULL);
 	bool runTempDlg(pki_temp *temp);
 	bool alterTemp(pki_temp *temp);
-	void showContextMenu(QContextMenuEvent *e, const QModelIndex &index);
+	void fillContextMenu(QMenu *menu, const QModelIndex &index);
 	pki_base *getByName(QString desc);
 	QStringList getDescPredefs();
-
-    public slots:
 	void newItem();
-	void changeTemp();
 	void showPki(pki_base *pki);
 	void load();
-	void store();
-	void certFromTemp();
-	void reqFromTemp();
-	void alterTemp();
-	void duplicateTemp();
-    signals:
-	void newReq(pki_temp *);
-	void newCert(pki_temp *);
-
+	void store(QModelIndex index);
 };
 #endif
