@@ -16,13 +16,13 @@
 #include "widgets/ExportDialog.h"
 #include "widgets/MainWindow.h"
 #include "widgets/PwDialog.h"
+#include "widgets/RevocationList.h"
 #include "ui_TrustState.h"
 #include "ui_CaProperties.h"
 #include "ui_About.h"
-#include "ui_Revoke.h"
-#include <QtGui/QMessageBox>
-#include <QtGui/QContextMenuEvent>
-#include <QtGui/QAction>
+#include <QMessageBox>
+#include <QContextMenuEvent>
+#include <QAction>
 
 bool db_x509::treeview = true;
 
@@ -30,22 +30,44 @@ db_x509::db_x509(QString DBfile, MainWindow *mw)
 	:db_x509super(DBfile, mw)
 {
 	rootItem->setIntName("[x509 root]");
-	allHeaders <<
-		new dbheader(HD_cert_ca,	true, tr("CA"),
+	class_name = "certificates";
+	pkitype << x509;
+	updateHeaders();
+	loadContainer();
+}
+
+void db_x509::updateAfterCrlLoad(pki_x509 *pki)
+{
+	if (pki->revList.merged) {
+		updatePKI(pki);
+		pki->revList.merged = false;
+	}
+}
+
+void db_x509::updateAfterDbLoad()
+{
+	FOR_ALL_pki(pki, pki_x509) {
+		updateAfterCrlLoad(pki);
+	}
+}
+
+dbheaderList db_x509::getHeaders()
+{
+	dbheaderList h = db_x509super::getHeaders();
+	h <<	new dbheader(HD_cert_ca,	true, tr("CA"),
 			tr("reflects the basic Constraints extension")) <<
 		new dbheader(HD_cert_serial,	true, tr("Serial")) <<
 		new dbheader(HD_cert_md5fp,	false,tr("md5 fingerprint")) <<
 		new dbheader(HD_cert_sha1fp,	false,tr("sha1 fingerprint")) <<
+		new dbheader(HD_cert_sha256fp,	false,tr("sha256 fingerprint")) <<
 		new dbheader(HD_cert_notBefore, false,tr("Start date"),
 				tr("not Before")) <<
 		new dbheader(HD_cert_notAfter,	true, tr("Expiry date"),
 				tr("not After")) <<
 		new dbheader(HD_cert_trust,	false,tr("Trust state")) <<
-		new dbheader(HD_cert_revokation,true, tr("Revocation"));
-
-	class_name = "certificates";
-	pkitype << x509;
-	loadContainer();
+		new dbheader(HD_cert_revocation,false, tr("Revocation")) <<
+		new dbheader(HD_cert_crl_expire,true, tr("CRL Expiration"));
+	return h;
 }
 
 pki_base *db_x509::newPKI(db_header_t *)
@@ -92,7 +114,7 @@ QStringList db_x509::getSignerDesc()
 void db_x509::remFromCont(QModelIndex &idx)
 {
 	int row;
-	pki_base *pki = static_cast<pki_base*>(currentIdx.internalPointer());
+	pki_base *pki = static_cast<pki_base*>(idx.internalPointer());
 	pki_base *parent_pki = pki->getParent();
 	row = pki->row();
 	pki_x509 *child;
@@ -132,8 +154,6 @@ void db_x509::changeView()
 		return;
 
 	temproot = new pki_base();
-	mainwin->certView->hide();
-	mainwin->certView->setModel(NULL);
 	beginRemoveRows(QModelIndex(), 0, rows -1);
 	pki_base *pki = rootItem;
 	pki_base *parent;
@@ -158,8 +178,6 @@ void db_x509::changeView()
 		inToCont(pki);
 	}
 	delete temproot;
-	mainwin->certView->setModel(this);
-	mainwin->certView->show();
 }
 
 void db_x509::calcEffTrust()
@@ -168,6 +186,23 @@ void db_x509::calcEffTrust()
 		pki->calcEffTrust();
 }
 
+static bool recursiveSigning(pki_x509 *cert, pki_x509 *client)
+{
+	/* recursive signing check */
+	for (pki_x509 *s = cert->getSigner(); s; s = s->getSigner()) {
+		if (s == s->getSigner()) {
+			return false;
+		}
+		if (s == client) {
+			printf("Recursive signing: '%s' <-> '%s'\n",
+				CCHAR(s->getIntName()),
+				CCHAR(cert->getIntName()));
+			return true;
+		}
+	}
+	return false;
+}
+
 void db_x509::inToCont(pki_base *pki)
 {
 	pki_x509 *cert = (pki_x509*)pki;
@@ -175,67 +210,64 @@ void db_x509::inToCont(pki_base *pki)
 	cert->delSigner(cert->getSigner());
 	findSigner(cert);
 	pki_base *root = cert->getSigner();
-	if (!treeview || root == cert)
+	if (!treeview || root == cert || root == NULL)
 		root = rootItem;
 
-	insertChild(root, pki);
-	/* search for dangling certificates, which signer this is */
-	FOR_ALL_pki(client, pki_x509) {
-		if (client->getSigner() == NULL) {
-			if (client->verify(cert) && treeview ) {
-				int row = client->row();
-				pki_x509 *s;
-				/* recursive signing check */
-				for (s = cert; s; s = s->getSigner()) {
-					if (s == s->getSigner()) {
-						s = NULL;
-						break;
-					}
-					if (s == client) {
-						printf("Recursive signing: '%s' <-> '%s'\n",
-									CCHAR(client->getIntName()),
-									CCHAR(cert->getIntName()));
-						break;
-					}
-				}
-				if (s)
-					continue;
-				beginRemoveRows(QModelIndex(), row, row);
-				rootItem->takeChild(client);
-				endRemoveRows();
+	insertChild(root, cert);
 
-				insertChild(pki, client);
-				client = (pki_x509*)rootItem;
-			}
+	QList<pki_x509 *> childs;
+	/* Search for another certificate (name and key)
+	 * and use its childs if we are newer */
+	FOR_ALL_pki(client, pki_x509) {
+		if (client == cert)
+			continue;
+		if (!client->compareNameAndKey(cert))
+			continue;
+		if (cert->getNotAfter() < client->getNotAfter())
+			continue;
+		foreach(pki_base *_child, client->childItems) {
+			pki_x509 *child = static_cast<pki_x509*>(_child);
+			child->delSigner(client);
+			childs << child;
 		}
 	}
+	/* Search rootItem childs, whether they are ours */
+	foreach(pki_base *_child, rootItem->childItems) {
+		pki_x509 *child = static_cast<pki_x509*>(_child);
+		if (child == cert || child->getSigner() == child)
+			continue;
+		if (child->verify_only(cert))
+			childs << child;
+	}
+	/* move collected childs to us */
+	foreach(pki_x509 *child, childs) {
+		int row;
+		if (recursiveSigning(cert, child))
+			continue;
+		if (!child->verify(cert))
+			continue;
+		row = child->row();
+		beginRemoveRows(index(child), row, row);
+		child->getParent()->takeChild(child);
+		endRemoveRows();
+		if (treeview)
+			insertChild(cert, child);
+		else
+			insertChild(rootItem, child);
+	}
 	findKey(cert);
 	pki_key *pub = cert->getPubKey();
 	if (pub) {
 		QList<pki_x509super *> reqs = mainwin->reqs->findByPubKey(pub);
 		delete pub;
 
-		foreach (pki_x509super *r, reqs) {
+		foreach(pki_x509super *r, reqs) {
 			((pki_x509req*)r)->setDone();
 		}
 	}
 	calcEffTrust();
 }
 
-
-QList<pki_x509*> db_x509::getIssuedCerts(const pki_x509 *issuer)
-{
-	QList<pki_x509*> c;
-	c.clear();
-	if (!issuer)
-		return c;
-	FOR_ALL_pki(pki, pki_x509) {
-		if (pki->getSigner() == issuer)
-			c.append(pki);
-	}
-	return c;
-}
-
 pki_x509 *db_x509::getBySubject(const x509name &xname, pki_x509 *last)
 {
 	bool lastfound = false;
@@ -254,29 +286,12 @@ pki_x509 *db_x509::getBySubject(const x509name &xname, pki_x509 *last)
 	return NULL;
 }
 
-void db_x509::revokeCert(const x509rev &revok, const pki_x509 *iss)
-{
-	pki_x509 *crt = getByIssSerial(iss, revok.getSerial());
-	if (crt)
-		crt->setRevoked(revok.getDate());
-}
-
-pki_x509 *db_x509::getByIssSerial(const pki_x509 *issuer, const a1int &a)
-{
-	if (!issuer ) return NULL;
-	FOR_ALL_pki(pki, pki_x509) {
-		if ((pki->getSigner() == issuer) && (a == pki->getSerial()))
-			return pki;
-	}
-	return NULL;
-}
-
 void db_x509::writeAllCerts(const QString fname, bool onlyTrusted)
 {
 	bool append = false;
 	FOR_ALL_pki(pki, pki_x509) {
 		if (onlyTrusted && pki->getTrust() != 2) continue;
-		pki->writeCert(fname.toAscii(), true, append);
+		pki->writeCert(fname.toLatin1(), true, append);
 		append = true;
 	}
 }
@@ -314,18 +329,16 @@ a1int db_x509::getUniqueSerial(pki_x509 *signer)
 {
 	// returnes an unused unique serial
 	a1int serial;
-	bool dup;
-	do {
-		dup = false;
+	x509rev rev;
+	while (true) {
 		serial = signer->getIncCaSerial();
-		FOR_ALL_pki(pki, pki_x509)
-			if (pki->getSigner() == signer)  {
-				if (serial == pki->getSerial()) {
-					dup = true;
-					break;
-				}
-			}
-	} while (dup);
+		rev.setSerial(serial);
+		if (signer->revList.contains(rev))
+			continue;
+		if (signer->getBySerial(serial))
+			continue;
+		break;
+	}
 	if (!signer->usesRandomSerial())
 		updatePKI(signer);
 	return serial;
@@ -358,6 +371,9 @@ pki_base *db_x509::insert(pki_base *item)
 	if ( ++serial > cert->getCaSerial()) {
 		cert->setCaSerial(serial);
 	}
+	if (mainwin->crls) {
+		mainwin->crls->updateRevocations(cert);
+	}
 	updatePKI(cert);
 	return cert;
 }
@@ -368,18 +384,6 @@ void db_x509::load(void)
 	load_default(c);
 }
 
-void db_x509::loadPKCS12()
-{
-	load_pkcs12 l;
-	load_default(l);
-}
-
-void db_x509::loadPKCS7()
-{
-	load_pkcs7 l;
-	load_default(l);
-}
-
 pki_x509 *db_x509::get1SelectedCert()
 {
 	QModelIndexList indexes = mainwin->certView->getSelectedIndexes();
@@ -430,6 +434,18 @@ void db_x509::newCert(pki_temp *temp)
 	delete dlg;
 }
 
+void db_x509::newCert(pki_x509 *cert)
+{
+	NewX509 *dlg = new NewX509(mainwin);
+	emit connNewX509(dlg);
+	dlg->setCert();
+	dlg->defineCert(cert);
+	if (dlg->exec()) {
+		newCert(dlg);
+	}
+	delete dlg;
+}
+
 void db_x509::newCert(NewX509 *dlg)
 {
 	pki_x509 *cert = NULL;
@@ -485,12 +501,6 @@ void db_x509::newCert(NewX509 *dlg)
 	} else if (dlg->selfQASignRB->isChecked()){
 		Passwd pass;
 		pass_info p(XCA_TITLE, tr("Please enter the new hexadecimal secret number for the QA process."));
-#if 0
-		ui.passA->setValidator(new QRegExpValidator(QRegExp("[0-9a-fA-F]*"),
-				ui.passA));
-		ui.passB->setValidator(new QRegExpValidator(QRegExp("[0-9a-fA-F]*"),
-				ui.passB));
-#endif
 		if (PwDialog::execute(&p, &pass, true) != 1)
 			throw errorEx(tr("The QA process has been terminated by the user."));
 		signcert = cert;
@@ -588,141 +598,104 @@ void db_x509::showPki(pki_base *pki)
 	dlg = new CertDetail(mainwin);
 	if (dlg) {
 		dlg->setCert(crt);
-		connect( dlg->privKey, SIGNAL( doubleClicked(QString) ),
-			mainwin->keys, SLOT( showItem(QString) ));
-		connect( dlg->signature, SIGNAL( doubleClicked(QString) ),
+		connect(dlg->privKey, SIGNAL(doubleClicked(QString)),
+			mainwin->keys, SLOT(showItem(QString)));
+		connect(dlg->signature, SIGNAL(doubleClicked(QString)),
 			this, SLOT( showItem(QString) ));
 		dlg->exec();
 		delete dlg;
 	}
 }
 
-void db_x509::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
+void db_x509::store(QModelIndex idx)
 {
-	QMenu *menu = new QMenu(mainwin);
-	QMenu *subExport, *subCa;
-	QAction *itemReq, *itemRevoke, *itemExtend, *itemTrust, *action;
-	QList<QAction*> scardItems;
-
-	bool parentCanSign, canSign, hasScard;
-	currentIdx = index;
-	pki_key *privkey;
-
-	pki_x509 *cert = static_cast<pki_x509*>(index.internalPointer());
-
-	menu->addAction(tr("New Certificate"), this, SLOT(newItem()));
-	menu->addAction(tr("Import"), this, SLOT(load()));
-	menu->addAction(tr("Import PKCS#12"), this, SLOT(loadPKCS12()));
-	menu->addAction(tr("Import from PKCS#7"), this, SLOT(loadPKCS7()));
-	if (index != QModelIndex()) {
-		privkey = cert->getRefKey();
-		menu->addAction(tr("Rename"), this, SLOT(edit()));
-		menu->addAction(tr("Show Details"), this, SLOT(showItem()));
-		if (!privkey)
-			menu->addAction(tr("Extract public Key"),
-				this, SLOT(extractPubkey()));
-		subExport = menu->addMenu(tr("Export"));
-		subExport->addAction(tr("Clipboard"), this,
-					SLOT(pem2clipboard()));
-		subExport->addAction(tr("File"), this, SLOT(store()));
-		itemReq = subExport->addAction(tr("Request"),
-				this, SLOT(toRequest()));
-		scardItems += subExport->addAction(tr("Security token"),
-				this, SLOT(toToken()));
-		scardItems += subExport->addAction(tr("Other token"),
-				this, SLOT(toOtherToken()));
-		subExport->addAction(tr("Template"), this, SLOT(toTemplate()));
-		subExport->addAction(tr("OpenSSL config"),
-				this, SLOT(toOpenssl()));
-
-		menu->addAction(tr("Delete"), this, SLOT(delete_ask()));
-		scardItems += menu->addAction(
-				tr("Delete from Security token"),
-				this, SLOT(deleteFromToken()));
-		itemTrust = menu->addAction(tr("Trust"), this,SLOT(setTrust()));
-		menu->addSeparator();
-		subCa = menu->addMenu(tr("CA"));
-		subCa->addAction(tr("Properties"), this, SLOT(caProperties()));
-		subCa->addAction(tr("Generate CRL"), this, SLOT(genCrl()));
-#if 0
-		QMenu *subP7 = menu->addMenu(tr("PKCS#7"));
-		subP7->addAction(tr("Sign"), this, SLOT(signP7()));
-		subP7->addAction(tr("Encrypt"), this, SLOT(encryptP7()));
-#endif
-		menu->addSeparator();
-		itemExtend = menu->addAction(tr("Renewal"),
-				this, SLOT(extendCert()));
-		if (cert->isRevoked()) {
-			itemRevoke = menu->addAction(tr("Unrevoke"),
-				this, SLOT(unRevoke()));
-			itemTrust->setEnabled(false);
-		} else {
-			itemRevoke = menu->addAction(tr("Revoke"),
-				this, SLOT(revoke()));
-		}
-		parentCanSign = (cert->getSigner() && cert->getSigner()->canSign()
-					&& (cert->getSigner() != cert));
-		canSign = cert->canSign();
-		hasScard = pkcs11::loaded();
-
-		itemRevoke->setEnabled(parentCanSign);
-		itemExtend->setEnabled(parentCanSign);
-		subCa->setEnabled(canSign);
-		itemReq->setEnabled(privkey && privkey->isPrivKey());
-
-		foreach(action, scardItems)
-			action->setEnabled(hasScard);
-#if 0
-		subP7->setEnabled(privkey);
-#endif
-	}
-	contextMenu(e, menu);
-	currentIdx = QModelIndex();
-	return;
+	QModelIndexList l;
+	l << idx;
+	store(l);
 }
 
-
-#define P7_ONLY 0
-#define P7_CHAIN 1
-#define P7_TRUSTED 2
-#define P7_ALL 3
-
-void db_x509::store()
+void db_x509::store(QModelIndexList list)
 {
 	QStringList filt;
-	bool pkcs8 = false, append;
+	bool append, chain;
+	QList<exportType> types, usual;
 
-	if (!currentIdx.isValid())
+	if (list.size() == 0)
 		return;
 
-	pki_x509 *crt = static_cast<pki_x509*>(currentIdx.internalPointer());
+	pki_x509 *crt = static_cast<pki_x509*>(list[0].internalPointer());
 	pki_x509 *oldcrt = NULL;
 	if (!crt)
 		return;
-	pki_key *privkey = crt->getRefKey();
-	QString fn = mainwin->getPath() + QDir::separator() +
-			crt->getUnderlinedName() + ".crt";
-	ExportCert *dlg = new ExportCert(mainwin, fn,
-		(privkey && privkey->isPrivKey()) && !privkey->isToken());
-	dlg->image->setPixmap(*MainWindow::certImg);
-	int dlgret = dlg->exec();
 
-	if (!dlgret) {
+	pki_key *privkey = crt->getRefKey();
+	pki_evp *pkey;
+	chain = crt->getSigner() && crt->getSigner() != crt;
+
+	usual <<
+		exportType(exportType::PEM, "crt", "PEM") <<
+		exportType(exportType::PKCS7, "p7b", "PKCS #7");
+
+	types << exportType(exportType::DER, "cer", "DER");
+
+	if (list.size() > 1) {
+		usual <<
+			exportType(exportType::PEM_selected, "pem",
+				"PEM selected") <<
+			exportType(exportType::PKCS7_selected, "pem",
+				"PKCS7 selected");
+	}
+	if (chain) {
+		types <<
+			exportType(exportType::PEM_chain, "pem",
+				tr("PEM chain")) <<
+			exportType(exportType::PKCS7_chain, "p7b",
+				tr("PKCS #7 chain"));
+	}
+
+	if (privkey && privkey->isPrivKey() && !privkey->isToken()) {
+		if (chain) {
+			usual << exportType(exportType::PKCS12_chain, "p12",
+				tr("PKCS #12 chain"));
+			types << exportType(exportType::PKCS12, "p12",
+				"PKCS #12");
+		} else {
+			usual << exportType(exportType::PKCS12, "p12",
+				"PKCS #12");
+		}
+		types <<
+			exportType(exportType::PEM_cert_key, "pem",
+				tr("PEM + key")) <<
+			exportType(exportType::PEM_cert_pk8, "pem",
+				"PEM + PKCS#8");
+	}
+	types << exportType() <<
+		exportType(exportType::PKCS7_trusted, "p7b",
+			tr("PKCS #7 trusted")) <<
+		exportType(exportType::PKCS7_all, "p7b",
+			tr("PKCS #7 all")) <<
+		exportType(exportType::PEM_trusted, "pem",
+			tr("PEM trusted")) <<
+		exportType(exportType::PEM_all, "pem",
+			tr("PEM all"));
+
+	types = usual << exportType() << types;
+	ExportDialog *dlg = new ExportDialog(mainwin, tr("Certificate export"),
+		tr("X509 Certificates ( *.pem *.cer *.crt *.p12 *.p7b )"), crt,
+		MainWindow::certImg, types);
+	if (!dlg->exec()) {
 		delete dlg;
 		return;
 	}
 	QString fname = dlg->filename->text();
-	if (fname == "") {
-		delete dlg;
-		return;
-	}
-	mainwin->setPath(fname.mid(0, fname.lastIndexOf(QRegExp("[/\\\\]")) ));
+	enum exportType::etype type = dlg->type();
+	delete dlg;
 	try {
-		switch (dlg->exportFormat->currentIndex()) {
-		case 0: // PEM
-			crt->writeCert(fname,true,false);
+		switch (type) {
+		case exportType::PEM:
+			crt->writeCert(fname, true, false);
 			break;
-		case 1: // PEM with chain
+		case exportType::PEM_chain:
 			append = false;
 			while(crt && crt != oldcrt) {
 				crt->writeCert(fname, true, append);
@@ -731,60 +704,65 @@ void db_x509::store()
 				crt = crt->getSigner();
 			}
 			break;
-		case 2: // PEM all trusted Certificates
+		case exportType::PEM_selected:
+			append = false;
+			foreach(QModelIndex idx, list) {
+				crt = static_cast<pki_x509*>(idx.internalPointer());
+				crt->writeCert(fname, true, append);
+				append = true;
+			}
+			break;
+		case exportType::PEM_trusted:
 			writeAllCerts(fname,true);
 			break;
-		case 3: // PEM all Certificates
+		case exportType::PEM_all:
 			writeAllCerts(fname,false);
 			break;
-		case 4: // DER
+		case exportType::DER:
 			crt->writeCert(fname,false,false);
 			break;
-		case 5: // P7 lonely
-			writePKCS7(crt,fname, P7_ONLY);
+		case exportType::PKCS7:
+		case exportType::PKCS7_chain:
+		case exportType::PKCS7_trusted:
+		case exportType::PKCS7_selected:
+		case exportType::PKCS7_all:
+			writePKCS7(crt, fname, type, list);
 			break;
-		case 6: // P7
-			writePKCS7(crt,fname, P7_CHAIN);
+		case exportType::PKCS12:
+			writePKCS12(crt, fname,false);
 			break;
-		case 7: // P7
-			writePKCS7(crt,fname, P7_TRUSTED);
+		case exportType::PKCS12_chain:
+			writePKCS12(crt, fname,true);
 			break;
-		case 8: // P7
-			writePKCS7(crt,fname, P7_ALL);
-			break;
-		case 9: // P12
-			writePKCS12(crt,fname,false);
-			break;
-		case 10: // P12 + cert chain
-			writePKCS12(crt,fname,true);
-			break;
-		case 12: // Certificate and Key in PKCS8 format
-			pkcs8 = true;
-		case 11: // Certificate and Key in PEM format for apache
-			pki_evp *privkey = (pki_evp *)crt->getRefKey();
-			if (!privkey || privkey->isPubKey()) {
+		case exportType::PEM_cert_pk8:
+		case exportType::PEM_cert_key:
+			pkey = (pki_evp *)crt->getRefKey();
+			if (!pkey || pkey->isPubKey()) {
 				XCA_WARN(tr("There was no key found for the Certificate: '%1'").
 					arg(crt->getIntName()));
 				return;
 			}
-			if (privkey->isToken()) {
+			if (pkey->isToken()) {
 				XCA_WARN(tr("Not possible for a token key: '%1'").
 					arg(crt->getIntName()));
                                 return;
                         }
 
-			if (pkcs8) {
-				privkey->writePKCS8(fname, NULL, NULL, true);
+			if (type == exportType::PEM_cert_pk8) {
+				pkey->writePKCS8(fname, EVP_des_ede3_cbc(),
+						PwDialog::pwCallback, true);
 			} else {
-				privkey->writeKey(fname, NULL, NULL, true);
+				pkey->writeKey(fname, NULL, NULL, true);
 			}
 			crt->writeCert(fname, true, true);
+			break;
+		default:
+			exit(1);
 		}
 	}
 	catch (errorEx &err) {
 		MainWindow::Error(err);
 	}
-	delete dlg;
 }
 
 
@@ -804,7 +782,7 @@ void db_x509::writePKCS12(pki_x509 *cert, QString s, bool chain)
 		}
 		if (s.isEmpty())
 			return;
-		s = QDir::convertSeparators(s);
+		s = nativeSeparator(s);
 		pki_pkcs12 *p12 = new pki_pkcs12(cert->getIntName(), cert, privkey);
 		pki_x509 *signer = cert->getSigner();
 		while ((signer != NULL ) && (signer != cert) && chain) {
@@ -820,15 +798,15 @@ void db_x509::writePKCS12(pki_x509 *cert, QString s, bool chain)
     }
 }
 
-void db_x509::writePKCS7(pki_x509 *cert, QString s, int type)
+void db_x509::writePKCS7(pki_x509 *cert, QString s, exportType::etype type,
+			QModelIndexList list)
 {
-    pki_pkcs7 *p7 = NULL;
-    QList<pki_base> list;
+	pki_pkcs7 *p7 = NULL;
 
-    try {
-		p7 =  new pki_pkcs7("");
+	try {
+		p7 = new pki_pkcs7("");
 		switch (type) {
-		case P7_CHAIN:
+		case exportType::PKCS7_chain:
 			while (cert != NULL) {
 				p7->addCert(cert);
 				if (cert->getSigner() == cert)
@@ -837,172 +815,113 @@ void db_x509::writePKCS7(pki_x509 *cert, QString s, int type)
 					cert = cert->getSigner();
 			}
 			break;
-		case P7_ONLY:
+		case exportType::PKCS7:
 			p7->addCert(cert);
 			break;
-		case P7_TRUSTED:
-		case P7_ALL:
+		case exportType::PKCS7_selected:
+			foreach(QModelIndex idx, list) {
+				cert = static_cast<pki_x509*>(idx.internalPointer());
+				p7->addCert(cert);
+			}
+			break;
+		case exportType::PKCS7_trusted:
+		case exportType::PKCS7_all:
 			FOR_ALL_pki(cer, pki_x509) {
-				if ((type == P7_ALL) || (cer->getTrust() == 2))
+				if ((type == exportType::PKCS7_all) ||
+				    (cer->getTrust() == 2))
 					p7->addCert(cer);
 			}
+			break;
+		default:
+			exit(1);
 		}
 		p7->writeP7(s, false);
-    }
-    catch (errorEx &err) {
+	}
+	catch (errorEx &err) {
 		MainWindow::Error(err);
-    }
-    if (p7 != NULL )
+	}
+	if (p7 != NULL )
 		delete p7;
 
 }
-# if 0
-void ::signP7()
-{
-	QStringList filt;
-    try {
-	pki_x509 *cert = (pki_x509 *)getSelected();
-	if (!cert) return;
-	pki_key *privkey = cert->getRefKey();
-	if (!privkey || privkey->isPubKey()) {
-		XCA_WARN(tr("There was no key found for the Certificate: ") +
-			cert->getIntName());
-		return;
-	}
-		filt.append("All Files ( * )");
-	QString s="";
-	QStringList slist;
-	Q3FileDialog *dlg = new Q3FileDialog(this,0,true);
-	dlg->setCaption(tr("Import Certificate signing request"));
-	dlg->setFilters(filt);
-	dlg->setMode( Q3FileDialog::ExistingFiles );
-        dlg->setDir(MainWindow::getPath());
-	if (dlg->exec()) {
-		slist = dlg->selectedFiles();
-		MainWindow::setPath(dlg->dirPath());
-        }
-	delete dlg;
-	pki_pkcs7 * p7 = new pki_pkcs7("");
-	for ( QStringList::Iterator it = slist.begin(); it != slist.end(); ++it ) {
-		s = *it;
-		s = QDir::convertSeparators(s);
-		p7->signFile(cert, s);
-		p7->writeP7((s + ".p7s"), true);
-	}
-	delete p7;
-    }
-    catch (errorEx &err) {
-	Qt::SocketError(err);
-    }
-}
 
-void CertView::encryptP7()
+void db_x509::manageRevocations(QModelIndex idx)
 {
-	QStringList filt;
-    try {
-	pki_x509 *cert = (pki_x509 *)getSelected();
-	if (!cert) return;
-	pki_key *privkey = cert->getRefKey();
-	if (!privkey || privkey->isPubKey()) {
-		XCA_WARN(tr("There was no key found for the Certificate: ") +
-			cert->getIntName()) ;
-		return;
-	}
-	filt.append("All Files ( * )");
-	QString s="";
-	QStringList slist;
-	Q3FileDialog *dlg = new Q3FileDialog(this,0,true);
-	dlg->setCaption(tr("Import Certificate signing request"));
-	dlg->setFilters(filt);
-	dlg->setMode( Q3FileDialog::ExistingFiles );
-	dlg->setDir(MainWindow::getPath());
-	if (dlg->exec()) {
-		slist = dlg->selectedFiles();
-		MainWindow::setPath(dlg->dirPath());
-	}
-	delete dlg;
-	pki_pkcs7 * p7 = new pki_pkcs7("");
-	for ( QStringList::Iterator it = slist.begin(); it != slist.end(); ++it ) {
-		s = *it;
-		s = QDir::convertSeparators(s);
-		p7->encryptFile(cert, s);
-		p7->writeP7((s + ".p7m"), true);
-	}
-	delete p7;
-    }
-    catch (errorEx &err) {
-		Qt::SocketError(err);
-    }
-}
-#endif
-
-void db_x509::setMultiTrust(QAbstractItemView* view)
-{
-	QItemSelectionModel *selectionModel = view->selectionModel();
-	QModelIndexList indexes = selectionModel->selectedIndexes();
-	QString items;
-
-	foreach(currentIdx, indexes) {
-		   setTrust();
-	}
-	currentIdx = QModelIndex();
-}
-
-void db_x509::deleteFromToken()
-{
-	if (!currentIdx.isValid())
-		return;
-	pki_x509 *pki = static_cast<pki_x509*>(currentIdx.internalPointer());
-	try {
-		pki->deleteFromToken();
-	} catch (errorEx &err) {
-		mainwin->Error(err);
-	}
-}
-
-void db_x509::setTrust()
-{
-	int state, newstate = 0;
-	Ui::TrustState ui;
-	pki_x509 *cert = static_cast<pki_x509*>(currentIdx.internalPointer());
+	pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
 	if (!cert)
 		return;
+	RevocationList *dlg = new RevocationList(mainwin);
+	dlg->setRevList(cert->revList, cert);
+	connect(dlg, SIGNAL(genCRL(pki_x509*)),
+		mainwin->crls, SLOT(newItem(pki_x509*)));
+	if (dlg->exec()) {
+		cert->setRevocations(dlg->getRevList());
+		updatePKI(cert);
+		emit columnsContentChanged();
+	}
+}
+
+void db_x509::setTrust(QModelIndexList indexes)
+{
+	int newstate = -1;
+	Ui::TrustState ui;
+	pki_x509 *cert = NULL;
+
+	if (indexes.size() == 0)
+		return;
+	if (indexes.size() == 1)
+		cert = static_cast<pki_x509*>(indexes[0].internalPointer());
+
 	QDialog *dlg = new QDialog(mainwin);
 	ui.setupUi(dlg);
-
 	ui.image->setPixmap(*MainWindow::certImg);
-	state = cert->getTrust();
-	if (cert->getSigner() == cert) {
-		if (state == 1)
-			state = 0;
-		ui.trust1->setDisabled(true);
+	if (cert) {
+		int state = cert->getTrust();
+		if (cert->getSigner() == cert) {
+			if (state == 1)
+				state = 0;
+			ui.trust1->setDisabled(true);
+		}
+		if (state == 0 ) ui.trust0->setChecked(true);
+		if (state == 1 ) ui.trust1->setChecked(true);
+		if (state == 2 ) ui.trust2->setChecked(true);
 	}
-	if (state == 0 ) ui.trust0->setChecked(true);
-	if (state == 1 ) ui.trust1->setChecked(true);
-	if (state == 2 ) ui.trust2->setChecked(true);
-	ui.certName->setText(cert->getIntName());
-	if (dlg->exec()) {
-		if (ui.trust0->isChecked()) newstate = 0;
-		if (ui.trust1->isChecked()) newstate = 1;
-		if (ui.trust2->isChecked()) newstate = 2;
-		if (newstate!=state) {
+	if (!dlg->exec()) {
+		delete dlg;
+		return;
+	}
+	if (ui.trust0->isChecked()) newstate = 0;
+	if (ui.trust1->isChecked()) newstate = 1;
+	if (ui.trust2->isChecked()) newstate = 2;
+	delete dlg;
+	if (newstate == -1) {
+		return;
+	}
+	foreach(QModelIndex idx, indexes) {
+		cert = static_cast<pki_x509*>(idx.internalPointer());
+		if (newstate != cert->getTrust()) {
 			cert->setTrust(newstate);
 			updatePKI(cert);
 		}
 	}
-	delete dlg;
 }
 
-void db_x509::extendCert()
+void db_x509::certRenewal(QModelIndexList indexes)
 {
 	pki_x509 *oldcert = NULL, *signer = NULL, *newcert =NULL;
 	pki_key *signkey = NULL;
 	a1time time;
 	a1int serial;
 	CertExtend *dlg = NULL;
+	x509rev r;
+	bool doRevoke = false;
+
+	if (indexes.size() == 0)
+		return;
+	QModelIndex idx = indexes[0];
 
 	try {
-		oldcert = static_cast<pki_x509*>(currentIdx.internalPointer());
+		oldcert = static_cast<pki_x509*>(idx.internalPointer());
 		if (!oldcert ||
 				!(signer = oldcert->getSigner()) ||
 				!(signkey = signer->getRefKey()) ||
@@ -1014,24 +933,37 @@ void db_x509::extendCert()
 			delete dlg;
 			return;
 		}
-		newcert = new pki_x509(oldcert);
-		serial = getUniqueSerial(signer);
+		if (dlg->revoke->isChecked()) {
+			Revocation *revoke = new Revocation(mainwin, indexes);
+			doRevoke = revoke->exec();
+			r = revoke->getRevocation();
+			delete revoke;
+		}
+		foreach(idx, indexes) {
+			oldcert = static_cast<pki_x509*>
+					(idx.internalPointer());
+			newcert = new pki_x509(oldcert);
+			serial = getUniqueSerial(signer);
+			newcert->setRevoked(x509rev());
 
-		// change date and serial
-		newcert->setSerial(serial);
-		newcert->setNotBefore(dlg->notBefore->getDate());
-		a1time a;
-		if (dlg->noWellDefinedExpDate->isChecked())
-			a.setUndefined();
-		else
-			a = dlg->notAfter->getDate();
+			// change date and serial
+			newcert->setSerial(serial);
+			newcert->setNotBefore(dlg->notBefore->getDate());
+			a1time a;
+			if (dlg->noWellDefinedExpDate->isChecked())
+				a.setUndefined();
+			else
+				a = dlg->notAfter->getDate();
 
-		newcert->setNotAfter(a);
+			newcert->setNotAfter(a);
 
-		// and finally sign the cert
-		newcert->sign(signkey, oldcert->getDigest());
-		newcert = (pki_x509 *)insert(newcert);
-		createSuccess(newcert);
+			// and finally sign the cert
+			newcert->sign(signkey, oldcert->getDigest());
+			newcert = (pki_x509 *)insert(newcert);
+			createSuccess(newcert);
+		}
+		if (doRevoke)
+			do_revoke(indexes, r);
 	}
 	catch (errorEx &err) {
 		MainWindow::Error(err);
@@ -1040,45 +972,98 @@ void db_x509::extendCert()
 	}
 	if (dlg)
 		delete dlg;
+	emit columnsContentChanged();
 }
 
 
-void db_x509::revoke()
+void db_x509::revoke(QModelIndexList indexes)
 {
-	pki_x509 *cert = static_cast<pki_x509*>(currentIdx.internalPointer());
-	if (!cert)
+	if (indexes.size() == 0)
 		return;
-	Ui::Revoke ui;
-	QDialog *revoke = new QDialog(mainwin, 0);
-	ui.setupUi(revoke);
-	ui.invalid->setNow();
-	ui.reason->addItems(x509rev::crlreasons());
+	Revocation *revoke = new Revocation(mainwin, indexes);
 	if (revoke->exec()) {
-		cert->setRevoked(true, ui.invalid->getDate(),
-					ui.reason->currentText());
-		updatePKI(cert);
+		do_revoke(indexes, revoke->getRevocation());
 	}
+	emit columnsContentChanged();
 }
 
-void db_x509::unRevoke()
+void db_x509::do_revoke(QModelIndexList indexes, const x509rev &r)
 {
-	pki_x509 *cert = static_cast<pki_x509*>(currentIdx.internalPointer());
+	pki_x509 *parent = NULL, *cert, *iss;
+	x509revList revlist;
+
+	foreach(QModelIndex idx, indexes) {
+		cert = static_cast<pki_x509*>(idx.internalPointer());
+		iss = cert->getSigner();
+		if (parent == NULL) {
+			parent = iss;
+		} else if (parent != iss) {
+			parent = NULL;
+			break;
+		}
+	}
+	if (!parent) {
+		qWarning("%s(%d): Certs have different/no signer\n",
+			 __func__, __LINE__);
+	}
+	foreach(QModelIndex idx, indexes) {
+		pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
+		x509rev rev(r);
+		rev.setSerial(cert->getSerial());
+		cert->setRevoked(rev);
+		revlist << rev;
+	}
+	parent->mergeRevList(revlist);
+	updatePKI(parent);
+}
+
+void db_x509::unRevoke(QModelIndexList indexes)
+{
+	pki_x509 *parent = NULL;
+	foreach(QModelIndex idx, indexes) {
+		pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
+		if (!cert)
+			return;
+		pki_x509 *iss = cert->getSigner();
+		if (parent == NULL) {
+			parent = iss;
+		} else if (parent != iss) {
+			parent = NULL;
+			break;
+		}
+	}
+	if (!parent) {
+		qWarning("%s(%d): Certs have different/no signer\n",
+			 __func__, __LINE__);
+	}
+	foreach(QModelIndex idx, indexes) {
+		pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
+		int i;
+		x509rev rev;
+
+		cert->setRevoked(x509rev());
+		rev.setSerial(cert->getSerial());
+		i = parent->revList.indexOf(rev);
+		if (i != -1)
+			parent->revList.takeAt(i);
+	}
+	updatePKI(parent);
+	emit columnsContentChanged();
+}
+
+void db_x509::toCertificate(QModelIndex index)
+{
+	pki_x509 *cert = static_cast<pki_x509*>(index.internalPointer());
 	if (!cert)
 		return;
-	cert->setRevoked(false);
-	updatePKI(cert);
+	if (!cert->getRefKey() && cert->getSigner() != cert)
+		extractPubkey(index);
+	newCert(cert);
 }
 
-void db_x509::genCrl()
+void db_x509::toRequest(QModelIndex idx)
 {
-	pki_x509 *cert = static_cast<pki_x509*>(currentIdx.internalPointer());
-	mainwin->crls->newItem(cert);
-}
-
-
-void db_x509::toRequest()
-{
-	pki_x509 *cert = static_cast<pki_x509*>(currentIdx.internalPointer());
+	pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
 	if (!cert)
 		return;
 
@@ -1095,9 +1080,9 @@ void db_x509::toRequest()
 	}
 }
 
-void db_x509::myToToken(bool alwaysSelect)
+void db_x509::toToken(QModelIndex idx, bool alwaysSelect)
 {
-	pki_x509 *cert = static_cast<pki_x509*>(currentIdx.internalPointer());
+	pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
 	if (!cert)
 		return;
 	try {
@@ -1107,23 +1092,15 @@ void db_x509::myToToken(bool alwaysSelect)
         }
 }
 
-void db_x509::toToken()
-{
-	myToToken(false);
-}
-
-void db_x509::toOtherToken()
-{
-	myToToken(true);
-}
-
-void db_x509::caProperties()
+void db_x509::caProperties(QModelIndex idx)
 {
 	Ui::CaProperties ui;
 	int i;
-	pki_x509 *cert = static_cast<pki_x509*>(currentIdx.internalPointer());
+
+	pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
 	if (!cert)
 		return;
+
 	QDialog *dlg = new QDialog(mainwin);
 	ui.setupUi(dlg);
 	ui.serial->setText(cert->getCaSerial().toHex());
@@ -1153,4 +1130,3 @@ void db_x509::caProperties()
 	}
 	delete dlg;
 }
-
