@@ -11,11 +11,15 @@
 
 #include "db_key.h"
 #include "pki_x509super.h"
-#include <QtCore/QList>
+#include <QList>
+#include <QMap>
 
 class db_x509name: public db_base
 {
 	Q_OBJECT
+
+	protected:
+		dbheaderList getHeaders();
 	public:
 		db_x509name(QString db, MainWindow *mw);
 };
@@ -24,17 +28,19 @@ class db_x509super: public db_x509name
 {
 	Q_OBJECT
 
+	protected:
+		dbheaderList getHeaders();
 	public:
 		db_x509super(QString db, MainWindow *mw);
 		pki_key *findKey(pki_x509super *ref);
 		QList<pki_x509super *> findByPubKey(pki_key *refkey);
+		void extractPubkey(QModelIndex index);
+		void toTemplate(QModelIndex index);
+		void toOpenssl(QModelIndex index) const;
 
 	public slots:
 		void delKey(pki_key *delkey);
 		void newKey(pki_key *newKey);
-		void extractPubkey();
-		void toTemplate();
-		void toOpenssl() const;
 };
 
 #endif
