@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2011 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2015 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -50,7 +50,8 @@ load_key::load_key()
 	:load_base()
 {
 	filter = QObject::tr("PKI Keys ( *.pem *.der *.key );;"
-			"PKCS#8 Keys ( *.p8 *.pk8 );;") + filter;
+			"PKCS#8 Keys ( *.p8 *.pk8 );;"
+			"SSH Public Keys ( *.pub );;") + filter;
 	caption = QObject::tr("Import RSA key");
 }
 
