@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2014 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2015 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -8,12 +8,17 @@
 #ifndef __MAIN_H
 #define __MAIN_H
 
-#include <QtGui/QApplication>
-#include <QtCore/QTranslator>
+#include <QApplication>
+#include <QAction>
+#include <QTranslator>
+#include <QLocale>
 #include "widgets/MainWindow.h"
 #ifdef WIN32
 #include <windows.h>
 #endif
+#include "entropy.h"
+
+extern char segv_data[1024];
 
 class XcaTranslator : public QTranslator
 {
@@ -23,7 +28,7 @@ public:
 	bool load(const QLocale &locale, const QString &filename,
 		const QString &dir)
 	{
-#if QT_VERSION >= 0x040800
+#if 0
 		return QTranslator::load(locale, filename, "_", dir, ".qm");
 #else
 		return QTranslator::load(QString("%1_%2").arg(filename).arg(locale.name()), dir);
@@ -37,17 +42,23 @@ class XCA_application : public QApplication
 
 private:
 	MainWindow *mainw;
-	XcaTranslator qtTr;
-	XcaTranslator xcaTr;
+	XcaTranslator *qtTr;
+	XcaTranslator *xcaTr;
+	static QLocale lang;
+	Entropy entropy;
+	static QList<QLocale> langAvail;
 
 public:
 	XCA_application(int &argc, char *argv[]);
+	virtual ~XCA_application();
 	void setMainwin(MainWindow *m);
+	void setupLanguage(QLocale lang);
+	static QLocale language() { return lang; }
+	static QFont tableFont;
+	static bool languageAvailable(QLocale l);
+	bool eventFilter(QObject *watched, QEvent *ev);
 
-protected:
-	bool event(QEvent *ev);
-
-signals:
-	void openFiles(QStringList &);
+public slots:
+	void switchLanguage(QAction* a);
 };
 #endif
