@@ -9,7 +9,7 @@
 #include "pki_crl.h"
 #include "func.h"
 #include "exception.h"
-#include <QtCore/QDir>
+#include <QDir>
 
 QPixmap *pki_crl::icon = NULL;
 
@@ -21,7 +21,7 @@ pki_crl::pki_crl(const QString name )
 	class_name="pki_crl";
 	pki_openssl_error();
 	dataVersion=1;
-	pkiType=revokation;
+	pkiType=revocation;
 }
 
 void pki_crl::fromPEM_BIO(BIO *bio, QString name)
@@ -43,18 +43,18 @@ QString pki_crl::getMsg(msg_type msg)
 	 * %1 will be replaced by the internal name of the CRL
 	 */
 	switch (msg) {
-	case msg_import: return tr("Successfully imported the revokation list '%1'");
-	case msg_delete: return tr("Delete the revokation list '%1'?");
-	case msg_create: return tr("Successfully created the revokation list '%1'");
+	case msg_import: return tr("Successfully imported the revocation list '%1'");
+	case msg_delete: return tr("Delete the revocation list '%1'?");
+	case msg_create: return tr("Successfully created the revocation list '%1'");
 	/* %1: Number of CRLs; %2: list of CRL names */
-	case msg_delete_multi: return tr("Delete the %1 revokation lists: %2?");
+	case msg_delete_multi: return tr("Delete the %1 revocation lists: %2?");
 	}
 	return pki_base::getMsg(msg);
 }
 
 void pki_crl::fload(const QString fname)
 {
-	FILE *fp = fopen(QString2filename(fname), "r");
+	FILE *fp = fopen_read(fname);
 	X509_CRL *_crl;
 	if (fp != NULL) {
 		_crl = PEM_read_X509_CRL(fp, NULL, NULL, NULL);
@@ -67,7 +67,7 @@ void pki_crl::fload(const QString fname)
 		if (pki_ign_openssl_error()) {
 			if (_crl)
 				X509_CRL_free(_crl);
-			throw errorEx(tr("Unable to load the revokation list in file %1. Tried PEM and DER formatted CRL.").arg(fname));
+			throw errorEx(tr("Unable to load the revocation list in file %1. Tried PEM and DER formatted CRL.").arg(fname));
 		}
 		if (crl)
 			X509_CRL_free(crl);
@@ -154,9 +154,9 @@ QByteArray pki_crl::toData()
 	return ba;
 }
 
-void pki_crl::addRev(const x509rev &xrev)
+void pki_crl::addRev(const x509rev &xrev, bool withReason)
 {
-	X509_CRL_add0_revoked(crl, xrev.get());
+	X509_CRL_add0_revoked(crl, xrev.get(withReason));
 	pki_openssl_error();
 }
 
@@ -168,6 +168,16 @@ void pki_crl::addV3ext(const x509v3ext &e)
 	pki_openssl_error();
 }
 
+bool pki_crl::visible()
+{
+	extList el;
+	if (pki_x509name::visible())
+		return true;
+	if (getSigAlg().contains(limitPattern))
+		return true;
+	el.setStack(crl->crl->extensions);
+	return el.search(limitPattern);
+}
 
 void pki_crl::sign(pki_key *key, const EVP_MD *md)
 {
@@ -188,7 +198,7 @@ void pki_crl::writeDefault(const QString fname)
 
 void pki_crl::writeCrl(const QString fname, bool pem)
 {
-	FILE *fp = fopen(QString2filename(fname), "w");
+	FILE *fp = fopen_write(fname);
 	if (fp != NULL) {
 		if (crl){
 			if (pem)
@@ -202,8 +212,9 @@ void pki_crl::writeCrl(const QString fname, bool pem)
 		fopen_error(fname);
 }
 
-BIO *pki_crl::pem(BIO *b)
+BIO *pki_crl::pem(BIO *b, int format)
 {
+	(void)format;
 	if (!b)
 		b = BIO_new(BIO_s_mem());
 	PEM_write_bio_X509_CRL(b, crl);
@@ -230,16 +241,18 @@ int pki_crl::numRev()
 {
 	if (crl && crl->crl && crl->crl->revoked)
 		return sk_X509_REVOKED_num(crl->crl->revoked);
-	else
-		return 0;
+	return 0;
 }
 
-x509rev pki_crl::getRev(int num)
+x509revList pki_crl::getRevList()
 {
-	x509rev ret;
-	if (crl && crl->crl && crl->crl->revoked) {
-		ret.set(sk_X509_REVOKED_value(crl->crl->revoked, num));
+	x509revList ret;
+	int i, num = numRev();
+
+	for (i=0; i<num; i++) {
+		x509rev r(sk_X509_REVOKED_value(crl->crl->revoked, i));
 		pki_openssl_error();
+		ret << r;
 	}
 	return ret;
 }
@@ -257,10 +270,10 @@ bool pki_crl::verify(pki_key *key)
 {
 	bool ret=false;
 	if (crl && crl->crl && key) {
-		ret = (X509_CRL_verify(crl , key->getPubKey()) == 1);
+		ret = (X509_CRL_verify(crl, key->getPubKey()) == 1);
 		pki_ign_openssl_error();
 	}
-	return ret ;
+	return ret;
 }
 
 void pki_crl::setCrlNumber(a1int num)
@@ -272,6 +285,14 @@ void pki_crl::setCrlNumber(a1int num)
 	pki_openssl_error();
 }
 
+a1int pki_crl::getCrlNumber()
+{
+	a1int num;
+	if (!getCrlNumber(&num))
+		num.set(0L);
+	return num;
+}
+
 bool pki_crl::getCrlNumber(a1int *num)
 {
 	int j;
