@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001 - 2011 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2015 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -7,13 +7,14 @@
 #ifndef __PKI_KEY_H
 #define __PKI_KEY_H
 
-#include <QtCore/QString>
-#include <QtGui/QProgressBar>
+#include <QString>
+#include <QProgressBar>
 #include <openssl/rsa.h>
 #include <openssl/bn.h>
 #include <openssl/pem.h>
 #include <openssl/evp.h>
 #include "pki_base.h"
+#include "builtin_curves.h"
 
 #define MAX_KEY_LENGTH 4096
 
@@ -25,13 +26,18 @@ class pki_key: public pki_base
 		EVP_PKEY *key;
 		QString BN2QString(BIGNUM *bn) const;
 		QString BNoneLine(BIGNUM *bn) const;
+		QByteArray SSH2publicQByteArray();
 
 	private:
+		BIGNUM *ssh_key_data2bn(QByteArray *ba, bool skip = false);
+		void ssh_key_QBA2data(QByteArray &ba, QByteArray *data);
+		void ssh_key_bn2data(BIGNUM *bn, QByteArray *data);
 		int ucount; // usage counter
 	public:
 		pki_key(const QString name = "");
 		pki_key(const pki_key *pk);
 		virtual ~pki_key();
+		static builtin_curves builtinCurves;
 		enum passType { ptCommon, ptPrivate, ptBogus, ptPin };
 
 		virtual EVP_PKEY *decryptKey() const
@@ -51,12 +57,9 @@ class pki_key: public pki_base
 			return NULL;
 		}
 		virtual bool isToken();
-		virtual QString getTypeString(void);
+		virtual QString getTypeString(void) const;
 		virtual QString getIntNameWithType(void);
-		virtual QList<int> possibleHashNids()
-		{
-			return QList<int>();
-		}
+		virtual QList<int> possibleHashNids();
 		virtual QString getMsg(msg_type msg);
 		virtual QString length();
 
@@ -76,6 +79,7 @@ class pki_key: public pki_base
 		{
 			return key;
 		}
+		BIO *pem(BIO *, int);
 		QVariant column_data(dbheader *hd);
 		QString modulus();
 		QString pubEx();
@@ -86,6 +90,8 @@ class pki_key: public pki_base
 		void d2i(QByteArray &ba);
 		void d2i_old(QByteArray &ba, int type);
 		QByteArray i2d();
+		EVP_PKEY *load_ssh2_key(FILE *fp);
+		void writeSSH2public(QString fname);
 };
 
 #endif
