@@ -1,13 +1,13 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
 
-#ifndef PKI_MULTI_H
-#define PKI_MULTI_H
+#ifndef __PKI_MULTI_H
+#define __PKI_MULTI_H
 
 #include <openssl/pem.h>
 #include "pki_x509.h"
@@ -17,6 +17,7 @@
 
 class pki_multi: public pki_base
 {
+		Q_OBJECT
 	protected:
 		QList<pki_base*> multi;
 	public:
@@ -24,7 +25,13 @@ class pki_multi: public pki_base
 		~pki_multi();
 		void fromPEM_BIO(BIO *bio, QString name);
 		void fload(const QString fname);
+		void probeAnything(const QString fname);
 		pki_base *pull();
+		int count() const
+		{
+			return multi.count();
+		}
+
 };
 
 #endif
