@@ -1,17 +1,18 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2009 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2010 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
-#ifndef PKI_PKCS7_H
-#define PKI_PKCS7_H
+#ifndef __PKI_PKCS7_H
+#define __PKI_PKCS7_H
 
 #include "pki_x509.h"
 
 class pki_pkcs7: public pki_base
 {
+		Q_OBJECT
 	friend class pki_x509;
 	protected:
 		PKCS7 *p7;
