@@ -20,14 +20,15 @@
 #include <openssl/rand.h>
 #include <openssl/bn.h>
 
-#include <QtGui/QProgressDialog>
-#include <QtGui/QApplication>
-#include <QtCore/QDir>
-#include <QtGui/QMessageBox>
-#include <QtCore/QThread>
+#include <QProgressDialog>
+#include <QApplication>
+#include <QDir>
+#include <QMessageBox>
+#include <QThread>
 #include <ltdl.h>
 
 QPixmap *pki_scard::icon[1] = { NULL };
+bool pki_scard::only_token_hashes = false;
 
 void pki_scard::init(void)
 {
@@ -114,25 +115,39 @@ EVP_PKEY *pki_scard::load_pubkey(pkcs11 &p11, CK_OBJECT_HANDLE object) const
 	}
 #ifndef OPENSSL_NO_EC
 	case CKK_EC: {
-		unsigned long s;
-		const unsigned char *p;
+		QByteArray ba;
+		EC_GROUP *group;
+		ASN1_OCTET_STRING *os;
+
 		EC_KEY *ec = EC_KEY_new();
 
 		pk11_attr_data grp(CKA_EC_PARAMS);
-                p11.loadAttribute(grp, object);
-		s = grp.getValue(&p);
-		EC_GROUP *group = d2i_ECPKParameters(NULL, &p, s);
+		p11.loadAttribute(grp, object);
+		ba = grp.getData();
+		group = (EC_GROUP *)
+			d2i_bytearray(D2I_VOID(d2i_ECPKParameters), ba);
+		pki_openssl_error();
+
 		EC_GROUP_set_asn1_flag(group, 1);
 		EC_KEY_set_group(ec, group);
 		pki_openssl_error();
 
 		pk11_attr_data pt(CKA_EC_POINT);
-                p11.loadAttribute(pt, object);
-		BIGNUM *bn = pt.getBignum();
-		BN_CTX *ctx = BN_CTX_new();
-		EC_POINT *point = EC_POINT_bn2point(group, bn, NULL, ctx);
+		p11.loadAttribute(pt, object);
+		ba = pt.getData();
+		os = (ASN1_OCTET_STRING *)
+			d2i_bytearray(D2I_VOID(d2i_ASN1_OCTET_STRING), ba);
+		pki_openssl_error();
+
+		BIGNUM *bn = BN_bin2bn(os->data, os->length, NULL);
+		pki_openssl_error();
+
+		EC_POINT *point = EC_POINT_bn2point(group, bn, NULL, NULL);
+		BN_free(bn);
+		ASN1_OCTET_STRING_free(os);
+		pki_openssl_error();
+
 		EC_KEY_set_public_key(ec, point);
-		BN_CTX_free(ctx);
 		pki_openssl_error();
 
 		pkey = EVP_PKEY_new();
@@ -217,16 +232,51 @@ void pki_scard::deleteFromToken()
 	deleteFromToken(slot);
 }
 
+pk11_attlist pki_scard::objectAttributesNoId(EVP_PKEY *pk, bool priv) const
+{
+	QByteArray ba;
+	RSA *rsa = pk->pkey.rsa;
+	DSA *dsa = pk->pkey.dsa;
+#ifndef OPENSSL_NO_EC
+	EC_KEY *ec = pk->pkey.ec;
+#endif
+
+	pk11_attlist attrs(pk11_attr_ulong(CKA_CLASS,
+			priv ? CKO_PRIVATE_KEY : CKO_PUBLIC_KEY));
+
+	switch (EVP_PKEY_type(pk->type)) {
+	case EVP_PKEY_RSA:
+		attrs << pk11_attr_ulong(CKA_KEY_TYPE, CKK_RSA) <<
+			pk11_attr_data(CKA_MODULUS, rsa->n, false) <<
+			pk11_attr_data(CKA_PUBLIC_EXPONENT, rsa->e, false);
+		break;
+	case EVP_PKEY_DSA:
+		attrs << pk11_attr_ulong(CKA_KEY_TYPE, CKK_DSA) <<
+			pk11_attr_data(CKA_PRIME, dsa->p, false) <<
+			pk11_attr_data(CKA_SUBPRIME, dsa->q, false) <<
+			pk11_attr_data(CKA_BASE, dsa->g, false);
+		break;
+#ifndef OPENSSL_NO_EC
+	case EVP_PKEY_EC:
+		ba = i2d_bytearray(I2D_VOID(i2d_ECPKParameters),
+				EC_KEY_get0_group(ec));
+
+		attrs << pk11_attr_ulong(CKA_KEY_TYPE, CKK_EC) <<
+			pk11_attr_data(CKA_EC_PARAMS, ba);
+		break;
+#endif
+	default:
+		throw errorEx(QString("Unkown Keytype %d").arg(pk->type));
+
+	}
+	return attrs;
+}
+
 pk11_attlist pki_scard::objectAttributes(bool priv) const
 {
-	unsigned long cka_class = priv ? CKO_PRIVATE_KEY : CKO_PUBLIC_KEY;
 
-	pk11_attlist attrs(pk11_attr_ulong(CKA_CLASS, cka_class));
+	pk11_attlist attrs = objectAttributesNoId(key, priv);
 	attrs << getIdAttr();
-
-	if (EVP_PKEY_type(key->type) == EVP_PKEY_RSA) {
-		attrs << pk11_attr_data(CKA_MODULUS, key->pkey.rsa->n, false);
-	}
 	return attrs;
 }
 
@@ -281,24 +331,18 @@ int pki_scard::renameOnToken(slotid slot, QString name)
 
 void pki_scard::store_token(slotid slot, EVP_PKEY *pkey)
 {
+	QByteArray ba;
+	RSA *rsa = pkey->pkey.rsa;
+	DSA *dsa = pkey->pkey.dsa;
+#ifndef OPENSSL_NO_EC
+	EC_KEY *ec = pkey->pkey.ec;
+#endif
 	pk11_attlist pub_atts;
 	pk11_attlist priv_atts;
 	QList<CK_OBJECT_HANDLE> objects;
 
-	if (EVP_PKEY_type(pkey->type) != EVP_PKEY_RSA)
-		throw errorEx(tr("only RSA keys can be stored on tokens"));
-
-	RSA *rsakey = pkey->pkey.rsa;
-
-	pub_atts <<
-		pk11_attr_ulong(CKA_CLASS, CKO_PUBLIC_KEY) <<
-		pk11_attr_ulong(CKA_KEY_TYPE, CKK_RSA) <<
-		pk11_attr_data(CKA_MODULUS, rsakey->n, false);
-
-	priv_atts <<
-		pk11_attr_ulong(CKA_CLASS, CKO_PRIVATE_KEY) <<
-		pk11_attr_ulong(CKA_KEY_TYPE, CKK_RSA) <<
-		pk11_attr_data(CKA_MODULUS, rsakey->n, false);
+	pub_atts = objectAttributesNoId(pkey, false);
+	priv_atts = objectAttributesNoId(pkey, true);
 
 	pkcs11 p11;
 	p11.startSession(slot, true);
@@ -313,32 +357,73 @@ void pki_scard::store_token(slotid slot, EVP_PKEY *pkey)
 	}
 	pk11_attr_data new_id = p11.findUniqueID(CKO_PUBLIC_KEY);
 
-	pub_atts <<
+	pub_atts << new_id <<
 		pk11_attr_bool(CKA_TOKEN, true) <<
 		pk11_attr_data(CKA_LABEL, getIntName().toUtf8()) <<
 		pk11_attr_bool(CKA_PRIVATE, false) <<
-		new_id <<
-		pk11_attr_data(CKA_PUBLIC_EXPONENT, rsakey->e, false) <<
 		pk11_attr_bool(CKA_WRAP, true) <<
 		pk11_attr_bool(CKA_ENCRYPT, true) <<
 		pk11_attr_bool(CKA_VERIFY, true);
 
-	priv_atts <<
+	priv_atts << new_id <<
 		pk11_attr_bool(CKA_TOKEN, true) <<
-		pk11_attr_bool(CKA_PRIVATE, true) <<
 		pk11_attr_data(CKA_LABEL, desc.toUtf8()) <<
-		new_id <<
-		pk11_attr_data(CKA_PUBLIC_EXPONENT, rsakey->e, false) <<
-		pk11_attr_data(CKA_PRIVATE_EXPONENT, rsakey->d, false) <<
-		pk11_attr_data(CKA_PRIME_1, rsakey->p, false) <<
-		pk11_attr_data(CKA_PRIME_2, rsakey->q, false) <<
-		pk11_attr_data(CKA_EXPONENT_1, rsakey->dmp1, false) <<
-		pk11_attr_data(CKA_EXPONENT_2, rsakey->dmq1, false) <<
-		pk11_attr_data(CKA_COEFFICIENT, rsakey->iqmp, false) <<
+		pk11_attr_bool(CKA_PRIVATE, true) <<
 		pk11_attr_bool(CKA_UNWRAP, true) <<
 		pk11_attr_bool(CKA_DECRYPT, true) <<
 		pk11_attr_bool(CKA_SIGN, true);
 
+	switch (EVP_PKEY_type(pkey->type)) {
+	case EVP_PKEY_RSA:
+		priv_atts <<
+		pk11_attr_data(CKA_PRIVATE_EXPONENT, rsa->d, false) <<
+		pk11_attr_data(CKA_PRIME_1, rsa->p, false) <<
+		pk11_attr_data(CKA_PRIME_2, rsa->q, false) <<
+		pk11_attr_data(CKA_EXPONENT_1, rsa->dmp1, false) <<
+		pk11_attr_data(CKA_EXPONENT_2, rsa->dmq1, false) <<
+		pk11_attr_data(CKA_COEFFICIENT, rsa->iqmp, false);
+		break;
+	case EVP_PKEY_DSA:
+		priv_atts << pk11_attr_data(CKA_VALUE, dsa->priv_key, false);
+		pub_atts << pk11_attr_data(CKA_VALUE, dsa->pub_key, false);
+		break;
+#ifndef OPENSSL_NO_EC
+	case EVP_PKEY_EC: {
+		/* Public Key */
+		BIGNUM *point;
+		int size;
+		unsigned char *buf;
+		ASN1_OCTET_STRING *os;
+		point = EC_POINT_point2bn(EC_KEY_get0_group(ec),
+			EC_KEY_get0_public_key(ec),
+			EC_KEY_get_conv_form(ec), NULL, NULL);
+
+		pki_openssl_error();
+		size = BN_num_bytes(point);
+		buf = (unsigned char *)OPENSSL_malloc(size);
+		check_oom(buf);
+		BN_bn2bin(point, buf);
+		os = ASN1_OCTET_STRING_new();
+		/* set0 -> ASN1_OCTET_STRING_free() also free()s buf */
+		ASN1_STRING_set0(os, buf, size);
+		ba = i2d_bytearray(I2D_VOID(i2d_ASN1_OCTET_STRING), os);
+		ASN1_OCTET_STRING_free(os);
+		BN_free(point);
+		pki_openssl_error();
+		pub_atts << pk11_attr_data(CKA_EC_POINT, ba);
+
+		/* Private key */
+		priv_atts << pk11_attr_data(CKA_VALUE,
+					EC_KEY_get0_private_key(ec));
+		break;
+	}
+#endif
+	default:
+		throw errorEx(QString("Unkown Keytype %d").arg(pkey->type));
+
+	}
+
+
 	tkInfo ti = p11.tokenInfo();
 	if (p11.tokenLogin(ti.label(), false).isNull())
 		throw errorEx(tr("PIN input aborted"));
@@ -347,11 +432,9 @@ void pki_scard::store_token(slotid slot, EVP_PKEY *pkey)
 	p11.createObject(priv_atts);
 
 	pub_atts.reset();
-	pub_atts <<
-		pk11_attr_ulong(CKA_CLASS, CKO_PUBLIC_KEY) <<
-                pk11_attr_ulong(CKA_KEY_TYPE, CKK_RSA) <<
-		new_id <<
-                pk11_attr_data(CKA_MODULUS, rsakey->n, false);
+
+	pub_atts = objectAttributesNoId(pkey, false);
+	pub_atts << new_id;
 
 	objs = p11.objectList(pub_atts);
 	if (objs.count() == 0)
@@ -363,25 +446,49 @@ void pki_scard::store_token(slotid slot, EVP_PKEY *pkey)
 QList<int> pki_scard::possibleHashNids()
 {
 	QList<int> nids;
-	int i;
 
-	for (i=0; i< mech_list.count(); i++) {
-		switch (mech_list[i]) {
-		case CKM_MD5_RSA_PKCS:    nids << NID_md5; break;
-		case CKM_DSA_SHA1:
+	if (!only_token_hashes)
+		return pki_key::possibleHashNids();
+
+	foreach(CK_MECHANISM_TYPE mechanism, mech_list) {
+		switch (EVP_PKEY_type(key->type)) {
+		case EVP_PKEY_RSA:
+			switch (mechanism) {
+			case CKM_MD5_RSA_PKCS:    nids << NID_md5; break;
+			case CKM_SHA1_RSA_PKCS:   nids << NID_sha1; break;
+			case CKM_SHA256_RSA_PKCS: nids << NID_sha256; break;
+			case CKM_SHA384_RSA_PKCS: nids << NID_sha384; break;
+			case CKM_SHA512_RSA_PKCS: nids << NID_sha512; break;
+			case CKM_RIPEMD160_RSA_PKCS: nids << NID_ripemd160; break;
+			}
+			break;
+		case EVP_PKEY_DSA:
+			switch (mechanism) {
+			case CKM_DSA_SHA1:        nids << NID_sha1; break;
+			}
+			break;
 #ifndef OPENSSL_NO_EC
-		case CKM_ECDSA_SHA1:
+		case EVP_PKEY_EC:
+			switch (mechanism) {
+			case CKM_ECDSA_SHA1:      nids << NID_sha1; break;
+			}
+			break;
 #endif
-		case CKM_SHA1_RSA_PKCS:   nids << NID_sha1; break;
-		case CKM_SHA256_RSA_PKCS: nids << NID_sha256; break;
-		case CKM_SHA384_RSA_PKCS: nids << NID_sha384; break;
-		case CKM_SHA512_RSA_PKCS: nids << NID_sha512; break;
-		case CKM_RIPEMD160_RSA_PKCS: nids << NID_ripemd160; break;
 		}
 	}
 	if (nids.count() == 0) {
-		nids << NID_md5 << NID_sha1 << NID_sha256 <<
-			NID_sha384 << NID_sha512 << NID_ripemd160;
+		switch (EVP_PKEY_type(key->type)) {
+		case EVP_PKEY_RSA:
+			nids << NID_md5 << NID_sha1 << NID_sha256 <<
+				NID_sha384 << NID_sha512 << NID_ripemd160;
+			break;
+		case EVP_PKEY_DSA:
+#ifndef OPENSSL_NO_EC
+		case EVP_PKEY_EC:
+#endif
+			nids << NID_sha1;
+			break;
+		}
 	}
 	return nids;
 }
@@ -474,24 +581,28 @@ public:
 	pk11_attr_data id;
 	QString name;
 	int size;
+	int curve_nid;
+	int method;
 	pkcs11 *p11;
 
 	void run()
 	{
 		try {
-			id = p11->generateRSAKey(name, size);
+			id = p11->generateKey(name, method, size, curve_nid);
 		} catch (errorEx &e) {
 			err = e;
 		}
 	}
 };
 
-void pki_scard::generateKey_card(slotid slot, int size, QProgressBar *bar)
+void pki_scard::generateKey_card(int type, slotid slot, int size,
+		int curve_nid, QProgressBar *bar)
 {
 	pk11_attlist atts;
 
 	pkcs11 p11;
 	p11.startSession(slot, true);
+	p11.getRandom();
 
 	tkInfo ti = p11.tokenInfo();
 
@@ -501,6 +612,22 @@ void pki_scard::generateKey_card(slotid slot, int size, QProgressBar *bar)
 	keygenThread kt;
 	kt.name = getIntName();
 	kt.size = size;
+	kt.curve_nid = curve_nid;
+	switch (type) {
+	case EVP_PKEY_RSA:
+		kt.method = CKM_RSA_PKCS_KEY_PAIR_GEN;
+		break;
+	case EVP_PKEY_DSA:
+		kt.method = CKM_DSA_KEY_PAIR_GEN;
+		break;
+#ifndef OPENSSL_NO_EC
+	case EVP_PKEY_EC:
+		kt.method = CKM_EC_KEY_PAIR_GEN;
+		break;
+#endif
+	default:
+		throw errorEx(tr("Illegal Key generation method"));
+	}
 	kt.p11 = &p11;
 	kt.start();
 	while (!kt.wait(20)) {
@@ -565,6 +692,9 @@ void pki_scard::fromData(const unsigned char *p, db_header_t *head )
 		mech_list << db::intFromData(ba);
 
 	d2i(ba);
+	if (!key || !key->pkey.ptr) {
+		throw errorEx(tr("Ignoring unsupported token key"));
+	}
 
 	if (ba.count() > 0) {
 		my_error(tr("Wrong Size %1").arg(ba.count()));
@@ -576,7 +706,7 @@ bool pki_scard::isPubKey() const
 	return false;
 }
 
-QString pki_scard::getTypeString(void)
+QString pki_scard::getTypeString(void) const
 {
 	return tr("Token %1").arg(pki_key::getTypeString());
 }
@@ -598,13 +728,16 @@ EVP_PKEY *pki_scard::decryptKey() const
 	}
 	pk11_attlist atts = objectAttributes(true);
 	QList<CK_OBJECT_HANDLE> priv_objects = p11->objectList(atts);
-	if (priv_objects.count() != 1)
+	if (priv_objects.count() != 1) {
+		delete p11;
 		throw errorEx(tr("Failed to find the key on the token"));
+	}
 	EVP_PKEY *pkey = p11->getPrivateKey(key, priv_objects[0]);
 
-	if (!pkey)
+	if (!pkey) {
 		delete p11;
-
+		throw errorEx(tr("Failed to initialize the key on the token"));
+	}
 	pki_openssl_error();
 	return pkey;
 }
@@ -657,3 +790,17 @@ QVariant pki_scard::getIcon(dbheader *hd)
 	return hd->id == HD_internal_name ? QVariant(*icon[0]) : QVariant();
 }
 
+bool pki_scard::visible()
+{
+	QStringList sl;
+	if (pki_base::visible())
+		return true;
+
+	sl << card_serial << card_manufacturer << card_model <<
+		card_label << slot_label << object_id;
+	foreach(QString s, sl) {
+		if (s.contains(limitPattern))
+			return true;
+	}
+	return false;
+}
