@@ -14,11 +14,13 @@
 #include "exception.h"
 #include "pass_info.h"
 #include "widgets/MainWindow.h"
-#include <QtGui/QMessageBox>
-#include <QtCore/QDir>
+#include <QMessageBox>
+#include <QDir>
 #include <openssl/rand.h>
 
 bool pki_x509::dont_colorize_expiries = false;
+bool pki_x509::disable_netscape = false;
+
 QPixmap *pki_x509::icon[6] = { NULL, NULL, NULL, NULL, NULL, NULL };
 
 pki_x509::pki_x509(X509 *c)
@@ -39,12 +41,11 @@ pki_x509::pki_x509(const pki_x509 *crt)
 	setRefKey(crt->getRefKey());
 	trust = crt->trust;
 	efftrust = crt->efftrust;
-	revoked = crt->revoked;
 	caSerial = crt->caSerial;
 	caTemplate = crt->caTemplate;
+	revocation = crt->revocation;
 	crlDays = crt->crlDays;
 	crlExpiry = crt->crlExpiry;
-	isrevoked = isrevoked;
 	pki_openssl_error();
 }
 
@@ -80,7 +81,7 @@ void pki_x509::fromPEM_BIO(BIO *bio, QString name)
 {
 	X509 *_cert;
 	_cert = PEM_read_bio_X509(bio, NULL, NULL, NULL);
-	openssl_error(name);
+	pki_openssl_error();
 	X509_free(cert);
 	cert = _cert;
 	autoIntName();
@@ -92,7 +93,7 @@ void pki_x509::fromPEM_BIO(BIO *bio, QString name)
 
 void pki_x509::fload(const QString fname)
 {
-	FILE *fp = fopen(QString2filename(fname), "r");
+	FILE *fp = fopen_read(fname);
 	X509 *_cert;
 	if (!fp) {
 		fopen_error(fname);
@@ -132,18 +133,15 @@ void pki_x509::init()
 	psigner = NULL;
 	trust = 0;
 	efftrust = 0;
-	revoked = a1time::now();
 	caSerial = 1;
 	caTemplate = "";
 	crlDays = 30;
-	crlExpiry = a1time::now();
+	crlExpiry.setUndefined();
 	class_name = "pki_x509";
 	cert = NULL;
-	isrevoked = false;
-	dataVersion = 3;
+	dataVersion = 4;
 	pkiType = x509;
 	randomSerial = false;
-	revoke_reason = "";
 }
 
 void pki_x509::setSerial(const a1int &serial)
@@ -158,9 +156,20 @@ void pki_x509::setSerial(const a1int &serial)
 a1int pki_x509::getSerial() const
 {
 	a1int a(X509_get_serialNumber(cert));
+	pki_openssl_error();
 	return a;
 }
 
+pki_x509 *pki_x509::getBySerial(const a1int &a) const
+{
+	foreach(pki_base *p, childItems) {
+		pki_x509 *pki = static_cast<pki_x509 *>(p);
+		if (a == pki->getSerial())
+			return pki;
+	}
+	return NULL;
+}
+
 #define SERIAL_LEN 8
 a1int pki_x509::getIncCaSerial()
 {
@@ -185,16 +194,6 @@ a1int pki_x509::hashInfo(const EVP_MD *md) const
 	return a;
 }
 
-a1int pki_x509::getQASerial(const a1int &secret) const
-{
-	ASN1_INTEGER *hold = cert->cert_info->serialNumber;
-	cert->cert_info->serialNumber = secret.get();
-	a1int ret = hashInfo(EVP_md5());
-	ASN1_INTEGER_free(cert->cert_info->serialNumber);
-	cert->cert_info->serialNumber = hold;
-	return ret;
-}
-
 void pki_x509::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 {
 	QString desc;
@@ -244,6 +243,7 @@ void pki_x509::d2i(QByteArray &ba)
 		X509_free(cert);
 		cert = c;
 	}
+	pki_openssl_error();
 }
 
 QByteArray pki_x509::i2d()
@@ -371,21 +371,18 @@ int pki_x509::renameOnToken(slotid slot, QString name)
 	return 1;
 }
 
-bool pki_x509::verifyQASerial(const a1int &secret) const
-{
-	return getQASerial(secret) == getSerial();
-}
-
 void pki_x509::setNotBefore(const a1time &a)
 {
 	a1time t(a);
 	X509_set_notBefore(cert, t.get_utc());
+	pki_openssl_error();
 }
 
 void pki_x509::setNotAfter(const a1time &a)
 {
 	a1time t(a);
 	X509_set_notAfter(cert, t.get_utc());
+	pki_openssl_error();
 }
 
 a1time pki_x509::getNotBefore() const
@@ -419,6 +416,7 @@ void pki_x509::setSubject(const x509name &n)
 	if (cert->cert_info->subject != NULL)
 		X509_NAME_free(cert->cert_info->subject);
 	cert->cert_info->subject = n.get();
+	pki_openssl_error();
 }
 
 void pki_x509::setIssuer(const x509name &n)
@@ -426,6 +424,7 @@ void pki_x509::setIssuer(const x509name &n)
 	if ((cert->cert_info->issuer) != NULL)
 		X509_NAME_free(cert->cert_info->issuer);
 	cert->cert_info->issuer = n.get();
+	pki_openssl_error();
 }
 
 bool pki_x509::addV3ext(const x509v3ext &e, bool skip_existing)
@@ -484,6 +483,7 @@ void pki_x509::sign(pki_key *signkey, const EVP_MD *digest)
 void pki_x509::fromData(const unsigned char *p, db_header_t *head)
 {
 	int version, size;
+	bool isRevoked = false;
 
 	version = head->version;
 	size = head->len - sizeof(db_header_t);
@@ -491,9 +491,17 @@ void pki_x509::fromData(const unsigned char *p, db_header_t *head)
 	QByteArray ba((const char*)p, size);
 
 	d2i(ba);
+	pki_openssl_error();
 	trust = db::intFromData(ba);
-	isrevoked = db::boolFromData(ba);
-	revoked.d2i(ba);
+	if (version < 4) {
+		a1time revoked;
+		isRevoked = db::boolFromData(ba);
+		revoked.d2i(ba);
+		if (isRevoked) {
+			revocation.setDate(revoked);
+			revocation.setSerial(getSerial());
+		}
+	}
 	caSerial.setHex(db::stringFromData(ba));
 	caTemplate = db::stringFromData(ba);
 	crlDays = db::intFromData(ba);
@@ -502,14 +510,27 @@ void pki_x509::fromData(const unsigned char *p, db_header_t *head)
 		randomSerial = db::boolFromData(ba);
 	else
 		randomSerial = false;
-	if (version > 2) {
+	if (version > 2)
 		crlNumber.setHex(db::stringFromData(ba));
-		revoke_reason = db::stringFromData(ba);
+	if (version > 2 && version < 4) {
+		// load own revocation info, to tell daddy about it
+		a1time invalDate;
+		QString revoke_reason = db::stringFromData(ba);
 		invalDate.d2i(ba);
+		if (isRevoked) {
+			revocation.setReason(revoke_reason);
+			revocation.setInvalDate(invalDate);
+		}
+	}
+	if (version > 3) {
+		x509revList curr(revList);
+		revList.fromBA(ba);
+		revList.merge(curr);
 	}
 	if (ba.count() > 0) {
 		my_error(tr("Wrong Size %1").arg(ba.count()));
 	}
+	pki_openssl_error();
 }
 
 
@@ -519,8 +540,7 @@ QByteArray pki_x509::toData()
 
 	ba += i2d(); // cert
 	ba += db::intToData(trust);
-	ba += db::boolToData(isrevoked);
-	ba += revoked.i2d(); // revokation date
+	// version 4: don't store isrevoked, revoked
 
 	// the serial if this is a CA
 	ba += db::stringToData(caSerial.toHex());
@@ -531,8 +551,8 @@ QByteArray pki_x509::toData()
 	ba += crlExpiry.i2d(); // last CRL date
 	ba += db::boolToData(randomSerial);
 	ba += db::stringToData(crlNumber.toHex());
-	ba += db::stringToData(revoke_reason);
-	ba += invalDate.i2d();
+	// version 4: don't store own revocation but client revocations
+	ba += revList.toBA();
 	pki_openssl_error();
 	return ba;
 }
@@ -546,9 +566,9 @@ void pki_x509::writeDefault(const QString fname)
 void pki_x509::writeCert(const QString fname, bool PEM, bool append)
 {
 	FILE *fp;
-	const char *p = "w";
+	const char *p = "wb";
 	if (append)
-		p = "a";
+		p = "ab";
 	fp = fopen(QString2filename(fname), p);
 	if (fp != NULL) {
 		if (cert){
@@ -563,8 +583,9 @@ void pki_x509::writeCert(const QString fname, bool PEM, bool append)
 		fopen_error(fname);
 }
 
-BIO *pki_x509::pem(BIO *b)
+BIO *pki_x509::pem(BIO *b, int format)
 {
+	(void)format;
 	if (!b)
 		b = BIO_new(BIO_s_mem());
 	PEM_write_bio_X509(b, cert);
@@ -579,13 +600,9 @@ bool pki_x509::cmpIssuerAndSerial(pki_x509 *refcert)
 
 }
 
-bool pki_x509::verify(pki_x509 *signer)
+bool pki_x509::verify_only(pki_x509 *signer)
 {
-	if (psigner == signer)
-		return true;
-	if ((psigner != NULL )||( signer == NULL))
-		return false;
-	X509_NAME *subject =  X509_get_subject_name(signer->cert);
+	X509_NAME *subject = X509_get_subject_name(signer->cert);
 	X509_NAME *issuer = X509_get_issuer_name(cert);
 	pki_openssl_error();
 	if (X509_NAME_cmp(subject, issuer)) {
@@ -598,13 +615,44 @@ bool pki_x509::verify(pki_x509 *signer)
 	}
 	int i = X509_verify(cert, pub);
 	pki_ign_openssl_error();
-	if (i>0) {
+	return i>0;
+}
+
+bool pki_x509::verify(pki_x509 *signer)
+{
+	if (psigner == signer)
+		return true;
+	if ((psigner != NULL) || (signer == NULL))
+		return false;
+	if (verify_only(signer)) {
+		int idx;
+		x509rev r;
+		r.setSerial(getSerial());
 		psigner = signer;
+		psigner->revList.merge(x509revList(revocation));
+		idx = psigner->revList.indexOf(r);
+		if (idx != -1)
+			revocation = psigner->revList[idx];
 		return true;
 	}
 	return false;
 }
 
+void pki_x509::setRevocations(const x509revList &rl)
+{
+	revList = rl;
+	x509rev rev;
+
+	foreach(pki_base *p, childItems) {
+		pki_x509 *pki = static_cast<pki_x509 *>(p);
+		rev.setSerial(pki->getSerial());
+		int idx = revList.indexOf(rev);
+		if (idx != -1)
+			pki->revocation = revList[idx];
+		else
+			pki->revocation = x509rev();
+	}
+}
 
 pki_key *pki_x509::getPubKey() const
 {
@@ -617,24 +665,56 @@ pki_key *pki_x509::getPubKey() const
 	return key;
 }
 
+bool pki_x509::compareNameAndKey(pki_x509 *other)
+{
+	int r;
+	X509_NAME *s1, *s2;
+	EVP_PKEY *pub1, *pub2;
+
+	if (!cert || !other->cert)
+		return false;
+	s1 = X509_get_subject_name(cert);
+	s2 = X509_get_subject_name(other->cert);
+	pki_openssl_error();
+	if (!s1 || !s2)
+		return false;
+	/* X509_NAME_cmp returns 0 if they match */
+	r = X509_NAME_cmp(s1, s2);
+	pki_openssl_error();
+	if (r)
+		return false;
+	pub1 = X509_get_pubkey(cert);
+	pub2 = X509_get_pubkey(other->cert);
+	pki_ign_openssl_error();
+	if (!pub1 || !pub2)
+		return false;
+	/* EVP_PKEY_cmp() return 1 if the keys match */
+	r = EVP_PKEY_cmp(pub1, pub2);
+	pki_openssl_error();
+	return r == 1;
+}
+
 void pki_x509::setPubKey(pki_key *key)
 {
-	 X509_set_pubkey(cert, key->getPubKey());
+	X509_set_pubkey(cert, key->getPubKey());
+	pki_openssl_error();
 }
 
 QString pki_x509::fingerprint(const EVP_MD *digest)
 {
-	int j;
 	QString fp="";
 	char zs[4];
-	unsigned int n;
+	unsigned int n, j;
 	unsigned char md[EVP_MAX_MD_SIZE];
-	X509_digest(cert, digest, md, &n);
+
 	pki_openssl_error();
-	for (j=0; j<(int)n; j++) {
-		sprintf(zs, "%02X%c",md[j], (j+1 == (int)n) ?'\0':':');
-		fp += zs;
+	if (X509_digest(cert, digest, md, &n)) {
+		for (j=0; j<n; j++) {
+			sprintf(zs, "%02X%c",md[j], (j+1 == n) ?'\0':':');
+			fp += zs;
+		}
 	}
+	pki_openssl_error();
 	return fp;
 }
 
@@ -645,6 +725,7 @@ bool pki_x509::checkDate()
 	n = a1time::now(),
 	b = getNotBefore();
 	a = getNotAfter();
+	pki_openssl_error();
 
 	if (!a.isValid() || !b.isValid())
 		return false;
@@ -668,11 +749,21 @@ x509v3ext pki_x509::getExtByNid(int nid)
 	extList el = getV3ext();
 	int i = el.idxByNid(nid);
 
+	try {
+		pki_openssl_error();
+	} catch(errorEx &err) {
+		XCA_WARN(err.getString());
+	}
 	if (i == -1)
 		return x509v3ext();
 	return el[i];
 }
 
+ASN1_OBJECT *pki_x509::sigAlg()
+{
+	return cert->sig_alg->algorithm;
+}
+
 pki_x509 *pki_x509::getSigner()
 {
 	return (pki_x509 *)psigner;
@@ -706,34 +797,16 @@ void pki_x509::setEffTrust(int t)
 
 bool pki_x509::isRevoked()
 {
-	return isrevoked ;
+	return revocation.isValid();
 }
 
-void pki_x509::setRevoked(bool rev, a1time inval, QString reason)
+void pki_x509::setRevoked(const x509rev &revok)
 {
-	if (rev) {
+	revocation = revok;
+	if (revok.isValid()) {
 		setEffTrust(0);
-		revoked = a1time::now();
-		pki_openssl_error();
-		revoke_reason = reason;
-		invalDate = inval;
+		setTrust(0);
 	}
-	isrevoked = rev;
-	pki_openssl_error();
-}
-
-a1time &pki_x509::getRevoked()
-{
-	return revoked;
-}
-
-void pki_x509::setRevoked(const a1time &when)
-{
-	isrevoked = true;
-	revoked = when;
-	setEffTrust(0);
-	setTrust(0);
-	pki_openssl_error();
 }
 
 int pki_x509::calcEffTrust()
@@ -770,18 +843,6 @@ void pki_x509::setCrlExpiry(const a1time &time)
 	pki_openssl_error();
 }
 
-x509rev pki_x509::getRev(bool reason)
-{
-	x509rev a;
-	a.setDate(getRevoked());
-	a.setSerial(getSerial());
-	if (reason) {
-		a.setReason(revoke_reason);
-		a.setInvalDate(invalDate);
-	}
-	return a;
-}
-
 bool pki_x509::caAndPathLen(bool *ca, a1int *pathlen, bool *hasLen)
 {
 	x509v3ext e = getExtByNid(NID_basic_constraints);
@@ -795,6 +856,7 @@ bool pki_x509::caAndPathLen(bool *ca, a1int *pathlen, bool *hasLen)
 	if (ca)
 		*ca = bc->ca;
 	BASIC_CONSTRAINTS_free(bc);
+	pki_openssl_error();
 	return true;
 }
 
@@ -812,17 +874,20 @@ QVariant pki_x509::column_data(dbheader *hd)
 			return QVariant(getNotAfter().toSortable());
 		case HD_cert_trust:
 			return QVariant(truststatus[getTrust()]);
-		case HD_cert_revokation:
-			if (isRevoked())
-				return QVariant(getRevoked().toSortable());
-			else if (canSign())
-				return QVariant(tr("CRL expires: %1").
-					arg(crlExpiry.toSortable()));
-			return QVariant();
+		case HD_cert_revocation:
+			return QVariant(isRevoked() ?
+				revocation.getDate().toSortable() : "");
+		case HD_cert_crl_expire:
+			if (canSign() && !crlExpiry.isUndefined())
+				return QVariant(crlExpiry.toSortable());
+			else
+				return QVariant();
 		case HD_cert_md5fp:
 			return QVariant(fingerprint(EVP_md5()));
 		case HD_cert_sha1fp:
 			return QVariant(fingerprint(EVP_sha1()));
+		case HD_cert_sha256fp:
+			return QVariant(fingerprint(EVP_sha256()));
 		case HD_cert_ca: {
 			a1int len;
 			bool ca, haslen;
@@ -869,15 +934,21 @@ QVariant pki_x509::getIcon(dbheader *hd)
 	return QVariant(*icon[pixnum]);
 }
 
-QString pki_x509::getSigAlg()
+bool pki_x509::visible()
 {
-	QString alg = OBJ_nid2ln(OBJ_obj2nid(cert->sig_alg->algorithm));
-	return alg;
-}
-
-const EVP_MD *pki_x509::getDigest()
-{
-	return EVP_get_digestbyobj(cert->sig_alg->algorithm);
+	if (pki_x509super::visible())
+		return true;
+	if (getIssuer().search(limitPattern))
+		return true;
+	if (fingerprint(EVP_md5()).contains(limitPattern))
+		return true;
+	if (fingerprint(EVP_sha1()).contains(limitPattern))
+		return true;
+	if (fingerprint(EVP_sha256()).contains(limitPattern))
+		return true;
+	if (getSerial().toHex().contains(limitPattern))
+		return true;
+	return false;
 }
 
 QVariant pki_x509::bg_color(dbheader *hd)
@@ -914,15 +985,17 @@ QVariant pki_x509::bg_color(dbheader *hd)
 				return QVariant(BG_YELLOW);
 			break;
 		}
-		case HD_cert_revokation:
+		case HD_cert_crl_expire:
 			if (canSign()) {
 				QDateTime crlwarn, crlex;
 				crlex = crlExpiry;
-				crlwarn = crlex.addSecs(-2 *60*60*24);
-				if (crlex < now)
-					return QVariant(BG_RED);
-				if (crlwarn < now || !crlex.isValid())
-					return QVariant(BG_YELLOW);
+				if (!crlExpiry.isUndefined()) {
+					crlwarn = crlex.addSecs(-2 *60*60*24);
+					if (crlex < now)
+						return QVariant(BG_RED);
+					if (crlwarn < now || !crlex.isValid())
+						return QVariant(BG_YELLOW);
+				}
 			}
 	}
 	return QVariant();
@@ -941,18 +1014,20 @@ void pki_x509::oldFromData(unsigned char *p, int size)
 		trust = intFromData(ba);
 		sRev = intFromData(ba);
 		if (sRev) {
-			if (version != 3) isrevoked = true;
-			revoked.d2i(ba);
-		}
-		else {
-			isrevoked = false;
-			revoked = a1time::now();
+			a1time r;
+			r.d2i(ba);
+			if (version != 3) {
+				revocation.setSerial(getSerial());
+				revocation.setDate(r);
+			}
+		} else {
+			revocation = x509rev();
 		}
 
 		if (version == 1) {
 			caTemplate="";
 			caSerial=1;
-			crlExpiry=a1time::now();
+			crlExpiry.setUndefined();
 			crlDays=30;
 		}
 
@@ -982,7 +1057,6 @@ void pki_x509::oldFromData(unsigned char *p, int size)
 	}
 	else { // old version
 		d2i(ba);
-		revoked = NULL;
 		trust = 1;
 		efftrust = 1;
 	}
@@ -995,4 +1069,3 @@ void pki_x509::oldFromData(unsigned char *p, int size)
 		my_error(tr("Wrong Size %1").arg(ba.count()));
 	}
 }
-
