@@ -26,8 +26,8 @@ class pki_x509 : public pki_x509super
 		Q_OBJECT
 	private:
 		pki_x509 *psigner;
-		a1time revoked, crlExpiry, invalDate;
-		bool isrevoked, randomSerial;
+		a1time crlExpiry;
+		bool randomSerial;
 		int trust;
 		int efftrust;
 		a1int caSerial;
@@ -35,11 +35,17 @@ class pki_x509 : public pki_x509super
 		int crlDays;
 		QString caTemplate;
 		X509 *cert;
-		QString revoke_reason;
 		void init();
+		x509rev revocation;
+
+	protected:
+		ASN1_OBJECT *sigAlg();
+
 	public:
 		static QPixmap *icon[6];
 		static bool dont_colorize_expiries;
+		static bool disable_netscape;
+		x509revList revList;
 		pki_x509(X509 *c);
 		pki_x509(const pki_x509 *crt);
 		pki_x509(const QString name = "");
@@ -51,8 +57,6 @@ class pki_x509 : public pki_x509super
 		void fromPEM_BIO(BIO *bio, QString name);
 		void writeDefault(const QString fname);
 		a1int hashInfo(const EVP_MD *md) const;
-		a1int getQASerial(const a1int &secret) const;
-		bool verifyQASerial(const a1int &secret) const;
 		void setSerial(const a1int &serial);
 		a1int getSerial() const;
 		void setNotBefore(const a1time &a);
@@ -71,6 +75,7 @@ class pki_x509 : public pki_x509super
 		bool canSign();
 		void writeCert(const QString fname, bool PEM, bool append = false);
 		bool verify(pki_x509 *signer);
+		bool verify_only(pki_x509 *signer);
 		pki_key *getPubKey() const;
 		void setPubKey(pki_key *key);
 		pki_x509 *getSigner();
@@ -90,9 +95,9 @@ class pki_x509 : public pki_x509super
 		void setEffTrust(int t);
 		void setRevoked(bool rev, a1time inval = a1time(),
 				QString reason = QString());
-		void setRevoked(const a1time &when);
-		a1time &getRevoked();
+		void setRevoked(const x509rev &revok);
 		bool isRevoked();
+		pki_x509 *getBySerial(const a1int &a) const;
 		int calcEffTrust();
 		a1int getIncCaSerial();
 		a1int getCaSerial()
@@ -103,9 +108,14 @@ class pki_x509 : public pki_x509super
 		{
 			caSerial = s;
 		}
-		a1int getIncCrlNumber()
+		a1int getCrlNumber()
 		{
-			return ++crlNumber;
+			return crlNumber;
+		}
+		void setCrlNumber(a1int n)
+		{
+			if (n > crlNumber)
+				crlNumber = n;
 		}
 		void setTemplate(QString s)
 		{
@@ -133,15 +143,17 @@ class pki_x509 : public pki_x509super
 		{
 			randomSerial = r;
 		}
+		x509rev getRevocation()
+		{
+			return revocation;
+		}
 		pk11_attlist objectAttributes();
 		void setCrlExpiry(const a1time &time);
 		bool hasExtension(int nid);
 		bool cmpIssuerAndSerial(pki_x509 *refcert);
+		bool visible();
 		void updateView();
-		x509rev getRev(bool reason = true);
-		QString getSigAlg();
 		x509v3ext getExtByNid(int nid);
-		const EVP_MD *getDigest();
 		QVariant column_data(dbheader *hd);
 		QVariant getIcon(dbheader *hd);
 		QByteArray i2d();
@@ -150,8 +162,13 @@ class pki_x509 : public pki_x509super
 		void deleteFromToken(slotid slot);
 		virtual QString getMsg(msg_type msg);
 		virtual int renameOnToken(slotid slot, QString name);
-		BIO *pem(BIO *);
+		BIO *pem(BIO *, int);
 		virtual QVariant bg_color(dbheader *hd);
+		void mergeRevList(x509revList l) {
+			revList.merge(l);
+		}
+		void setRevocations(const x509revList &rl);
+		bool compareNameAndKey(pki_x509 *other);
 };
 
 #endif
