@@ -14,7 +14,7 @@
 #include "exception.h"
 #include <openssl/bio.h>
 #include <openssl/err.h>
-#include <QtCore/QDir>
+#include <QDir>
 
 QPixmap *pki_x509req::icon[4] = { NULL, NULL, NULL, NULL };
 
@@ -113,7 +113,7 @@ void pki_x509req::fromPEM_BIO(BIO *bio, QString name)
 
 void pki_x509req::fload(const QString fname)
 {
-	FILE *fp = fopen(QString2filename(fname), "r");
+	FILE *fp = fopen_read(fname);
 	X509_REQ *_req;
 	int ret = 0;
 
@@ -206,6 +206,11 @@ x509name pki_x509req::getSubject() const
 	return x;
 }
 
+ASN1_OBJECT *pki_x509req::sigAlg()
+{
+	return request->sig_alg->algorithm;
+}
+
 void pki_x509req::setSubject(const x509name &n)
 {
 	if (request->req_info->subject != NULL)
@@ -236,7 +241,7 @@ void pki_x509req::writeDefault(const QString fname)
 
 void pki_x509req::writeReq(const QString fname, bool pem)
 {
-	FILE *fp = fopen(QString2filename(fname), "w");
+	FILE *fp = fopen_write(fname);
 	if (fp) {
 		if (request){
 			if (pem)
@@ -250,8 +255,9 @@ void pki_x509req::writeReq(const QString fname, bool pem)
 		fopen_error(fname);
 }
 
-BIO *pki_x509req::pem(BIO *b)
+BIO *pki_x509req::pem(BIO *b, int format)
 {
+	(void)format;
 	if (!b)
 		b = BIO_new(BIO_s_mem());
 	PEM_write_bio_X509_REQ(b, request);
@@ -384,13 +390,13 @@ ASN1_IA5STRING *pki_x509req::spki_challange()
 	return NULL;
 }
 
-static QString getAttribute(X509_REQ *req, int nid)
+QString pki_x509req::getAttribute(int nid)
 {
 	int n;
-	n = X509_REQ_get_attr_by_NID(req, nid, -1);
+	n = X509_REQ_get_attr_by_NID(request, nid, -1);
 	if (n == -1)
 		return QString("");
-	X509_ATTRIBUTE *att = X509_REQ_get_attr(req, n);
+	X509_ATTRIBUTE *att = X509_REQ_get_attr(request, n);
 	if (!att)
 		return QString("");
 	if (att->single)
@@ -411,9 +417,9 @@ QVariant pki_x509req::column_data(dbheader *hd)
 	case HD_req_signed:
 		return QVariant(done ? tr("Signed") : tr("Unhandled"));
 	case HD_req_unstr_name:
-		return getAttribute(request, NID_pkcs9_unstructuredName);
+		return getAttribute(NID_pkcs9_unstructuredName);
 	case HD_req_chall_pass:
-		return getAttribute(request, NID_pkcs9_challengePassword);
+		return getAttribute(NID_pkcs9_challengePassword);
 	}
 	return pki_x509super::column_data(hd);
 }
@@ -442,6 +448,17 @@ QVariant pki_x509req::getIcon(dbheader *hd)
 	return QVariant(*icon[pixnum]);
 }
 
+bool pki_x509req::visible()
+{
+	if (pki_x509super::visible())
+		return true;
+	if (getAttribute(NID_pkcs9_unstructuredName).contains(limitPattern))
+		return true;
+	if (getAttribute(NID_pkcs9_challengePassword).contains(limitPattern))
+		return true;
+	return false;
+}
+
 void pki_x509req::oldFromData(unsigned char *p, int size)
 {
 	QByteArray ba((const char *)p, size);
