@@ -50,6 +50,18 @@ void pki_x509super::delRefKey(pki_key *ref)
 	privkey = NULL;
 }
 
+QString pki_x509super::getSigAlg()
+{
+	QString alg = OBJ_nid2ln(OBJ_obj2nid(sigAlg()));
+	pki_openssl_error();
+	return alg;
+}
+
+const EVP_MD *pki_x509super::getDigest()
+{
+	return EVP_get_digestbyobj(sigAlg());
+}
+
 QVariant pki_x509super::column_data(dbheader *hd)
 {
 	if (hd->id == HD_x509key_name) {
@@ -57,7 +69,10 @@ QVariant pki_x509super::column_data(dbheader *hd)
 			return QVariant("");
 		return QVariant(privkey->getIntName());
 	}
-	if (hd->type == dbheader::hd_v3ext) {
+	if (hd->id == HD_x509_sigalg) {
+		return QVariant(getSigAlg());
+	}
+	if (hd->type == dbheader::hd_v3ext || hd->type == dbheader::hd_v3ext_ns) {
 		extList el = getV3ext();
 		int idx = el.idxByNid(hd->id);
 		if (idx == -1)
@@ -112,7 +127,7 @@ void pki_x509super::opensslConf(QString fname)
 		"%2").arg(name).arg(extensions).
 			arg(ASN1_STRING_get_default_mask(), 0, 16);
 
-	FILE *fp = fopen(QString2filename(fname),"w");
+	FILE *fp = fopen_write(fname);
 	if (fp == NULL) {
 		fopen_error(fname);
 		return;
@@ -121,6 +136,16 @@ void pki_x509super::opensslConf(QString fname)
 	fwrite_ba(fp, ba, fname);
 	fclose(fp);
 }
+
+bool pki_x509super::visible()
+{
+	if (pki_x509name::visible())
+		return true;
+	if (getSigAlg().contains(limitPattern))
+		return true;
+	return getV3ext().search(limitPattern);
+}
+
 // Start class  pki_x509name
 
 pki_x509name::pki_x509name(const QString name)
@@ -148,3 +173,10 @@ QVariant pki_x509name::column_data(dbheader *hd)
 	}
 	return pki_base::column_data(hd);
 }
+
+bool pki_x509name::visible()
+{
+	if (pki_base::visible())
+		return true;
+	return getSubject().search(limitPattern);
+}
