@@ -24,6 +24,7 @@ class pki_x509name : public pki_base
 	};
 	void autoIntName();
 	QVariant column_data(dbheader *hd);
+	bool visible();
 };
 
 class pki_x509super : public pki_x509name
@@ -31,6 +32,9 @@ class pki_x509super : public pki_x509name
 		Q_OBJECT
 	protected:
 		pki_key *privkey;
+		virtual ASN1_OBJECT *sigAlg() {
+			return NULL;
+		}
 	public:
 		pki_x509super(const QString name = "");
 		virtual ~pki_x509super();
@@ -46,15 +50,18 @@ class pki_x509super : public pki_x509name
 		{
 			return extList();
 		};
-		virtual QString getSigAlg()
+		virtual QString getSigAlg();
+		virtual const EVP_MD *getDigest();
+		virtual bool isSpki() const
 		{
-			return QString();
+			return false;
 		}
 		pki_key *getRefKey() const;
 		void setRefKey(pki_key *ref);
 		void delRefKey(pki_key *ref);
 		QVariant column_data(dbheader *hd);
 		void opensslConf(QString fname);
+		bool visible();
 };
 
 #endif
