@@ -8,8 +8,8 @@
 #ifndef __X509NAME_H
 #define __X509NAME_H
 
-#include <QtCore/QString>
-#include <QtCore/QStringList>
+#include <QString>
+#include <QStringList>
 #include <openssl/x509.h>
 
 class x509name
@@ -43,6 +43,7 @@ class x509name
 		QString getMostPopular() const;
 		QString taggedValues() const;
 		QString hash() const;
+		bool search(const QRegExp &pattern);
 };
 
 #endif
