@@ -8,8 +8,8 @@
 #ifndef __X509V3EXT_H
 #define __X509V3EXT_H
 
-#include <QtCore/QList>
-#include <QtCore/QStringList>
+#include <QList>
+#include <QStringList>
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 
@@ -19,6 +19,7 @@ class x509v3ext
 {
    private:
 	X509_EXTENSION *ext;
+	const ASN1_OBJECT *object() const;
    public:
 	x509v3ext();
 	x509v3ext(const X509_EXTENSION *n);
@@ -26,6 +27,8 @@ class x509v3ext
 	~x509v3ext();
 	x509v3ext &set(const X509_EXTENSION *n);
 	x509v3ext &create(int nid, const QString &et, X509V3_CTX *ctx = NULL);
+	x509v3ext &create_ia5(int nid, const QString &et,
+				X509V3_CTX *ctx = NULL);
 	x509v3ext &operator = (const x509v3ext &x);
 	// bool operator == (const x509v3ext &x) const;
 	QString getObject() const;
@@ -50,6 +53,10 @@ class x509v3ext
 	bool parse_bitstring(QString *single, QString *adv) const;
 	bool parse_sKeyId(QString *single, QString *adv) const;
 	bool parse_aKeyId(QString *single, QString *adv) const;
+	bool parse_inhibitAnyPolicy(QString *, QString *adv) const;
+	bool parse_policyConstraints(QString *, QString *adv) const;
+	bool parse_policyMappings(QString *, QString *adv) const;
+	bool parse_nameConstraints(QString *, QString *adv) const;
 };
 
 class extList : public QList<x509v3ext>
@@ -63,5 +70,6 @@ class extList : public QList<x509v3ext>
 	int idxByNid(int nid);
 	bool genConf(int nid, QString *single, QString *adv = NULL);
 	void genGenericConf(QString *adv);
+	bool search(const QRegExp &pattern);
 };
 #endif
