@@ -2,13 +2,12 @@
 
 XCA_DIR="`dirname $0`"
 XCA_DIR="`cd $XCA_DIR/.. && pwd`"
-XCA_BUILD="`pwd`"/xca_build
 
 LIBTOOL_DIR="libtool-2.2.6b"
 LIBTOOL_GZ="${LIBTOOL_DIR}".tar.gz
 LIBTOOL_DL="http://ftp.gnu.org/gnu/libtool/${LIBTOOL_GZ}"
 
-OPENSSL_DIR="openssl-1.0.1j"
+OPENSSL_DIR="openssl-1.0.2d"
 OPENSSL_GZ="${OPENSSL_DIR}".tar.gz
 OPENSSL_DL="http://www.openssl.org/source/${OPENSSL_GZ}"
 OPENSSL_PATCH="$XCA_DIR/misc/openssl-1.0.0-mingw32-cross.patch"
@@ -46,6 +45,7 @@ do_XCA()
   cd $XCA_BUILD
   $XCA_DIR/configure.w32
   make -j5 USE_HOSTTOOLS=no
+  cp setup*.exe ..
 )}
 
 if ! test -f db_dump.exe; then
@@ -62,6 +62,8 @@ if ! test -f mingwm10.dll; then
   fi
 fi
 
+
+XCA_BUILD="`pwd`"/xca_build
 export INSTALL_DIR=`pwd`/install
 
 do_openssl
