@@ -11,7 +11,7 @@
 #include "ui_ImportMulti.h"
 #include "lib/db_token.h"
 #include "lib/db_base.h"
-#include <QtCore/QList>
+#include <QList>
 
 class pki_x509;
 class pki_key;
