@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2014 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2015 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -10,20 +10,23 @@
 #include "MainWindow.h"
 #include "ImportMulti.h"
 #include "lib/Passwd.h"
+#include "lib/entropy.h"
 
 #include <openssl/rand.h>
 
-#include <QtGui/QApplication>
-#include <QtGui/QClipboard>
-#include <QtGui/QMessageBox>
-#include <QtGui/QLabel>
-#include <QtGui/QPushButton>
-#include <QtGui/QListView>
-#include <QtGui/QLineEdit>
-#include <QtGui/QTextBrowser>
-#include <QtGui/QStatusBar>
-#include <QtCore/QList>
-#include <QtGui/QInputDialog>
+#include <QApplication>
+#include <QClipboard>
+#include <QMessageBox>
+#include <QLabel>
+#include <QPushButton>
+#include <QListView>
+#include <QLineEdit>
+#include <QTextBrowser>
+#include <QStatusBar>
+#include <QList>
+#include <QTimer>
+#include <QMimeData>
+#include <QInputDialog>
 
 #include "lib/exception.h"
 #include "lib/pki_evp.h"
@@ -34,6 +37,7 @@
 #include "lib/pass_info.h"
 #include "lib/func.h"
 #include "lib/pkcs11.h"
+#include "lib/builtin_curves.h"
 #include "ui_About.h"
 #include "PwDialog.h"
 
@@ -54,131 +58,10 @@ NIDlist *MainWindow::dn_nid = NULL;
 NIDlist *MainWindow::aia_nid = NULL;
 
 QString MainWindow::mandatory_dn;
+QString MainWindow::explicit_dn;
+QString MainWindow::explicit_dn_default = QString("C,ST,L,O,OU,CN,emailAddress");
 
-static const int x962_curve_nids[] = {
-	NID_X9_62_prime192v1,
-	NID_X9_62_prime192v2,
-	NID_X9_62_prime192v3,
-	NID_X9_62_prime239v1,
-	NID_X9_62_prime239v2,
-	NID_X9_62_prime239v3,
-	NID_X9_62_prime256v1,
-
-	NID_X9_62_c2pnb163v1,
-	NID_X9_62_c2pnb163v2,
-	NID_X9_62_c2pnb163v3,
-	NID_X9_62_c2pnb176v1,
-	NID_X9_62_c2tnb191v1,
-	NID_X9_62_c2tnb191v2,
-	NID_X9_62_c2tnb191v3,
-	NID_X9_62_c2pnb208w1,
-	NID_X9_62_c2tnb239v1,
-	NID_X9_62_c2tnb239v2,
-	NID_X9_62_c2tnb239v3,
-	NID_X9_62_c2pnb272w1,
-	NID_X9_62_c2pnb304w1,
-	NID_X9_62_c2tnb359v1,
-	NID_X9_62_c2pnb368w1,
-	NID_X9_62_c2tnb431r1
-};
-
-static const int other_curve_nids[] = {
-	NID_sect113r1,
-	NID_sect113r2,
-	NID_sect131r1,
-	NID_sect131r2,
-	NID_sect163k1,
-	NID_sect163r1,
-	NID_sect163r2,
-	NID_sect193r1,
-	NID_sect193r2,
-	NID_sect233k1,
-	NID_sect233r1,
-	NID_sect239k1,
-	NID_sect283k1,
-	NID_sect283r1,
-	NID_sect409k1,
-	NID_sect409r1,
-	NID_sect571k1,
-	NID_sect571r1,
-
-	NID_secp112r1,
-	NID_secp112r2,
-	NID_secp128r1,
-	NID_secp128r2,
-	NID_secp160k1,
-	NID_secp160r1,
-	NID_secp160r2,
-	NID_secp192k1,
-	NID_secp224k1,
-	NID_secp224r1,
-	NID_secp256k1,
-	NID_secp384r1,
-	NID_secp521r1,
-
-	NID_wap_wsg_idm_ecid_wtls1,
-	NID_wap_wsg_idm_ecid_wtls3,
-	NID_wap_wsg_idm_ecid_wtls4,
-	NID_wap_wsg_idm_ecid_wtls5,
-	NID_wap_wsg_idm_ecid_wtls6,
-	NID_wap_wsg_idm_ecid_wtls7,
-	NID_wap_wsg_idm_ecid_wtls8,
-	NID_wap_wsg_idm_ecid_wtls9,
-	NID_wap_wsg_idm_ecid_wtls10,
-	NID_wap_wsg_idm_ecid_wtls11,
-	NID_wap_wsg_idm_ecid_wtls12,
-
-#if OPENSSL_VERSION_NUMBER >= 0x10002001L
-	NID_brainpoolP160r1,
-	NID_brainpoolP160t1,
-	NID_brainpoolP192r1,
-	NID_brainpoolP192t1,
-	NID_brainpoolP224r1,
-	NID_brainpoolP224t1,
-	NID_brainpoolP256r1,
-	NID_brainpoolP256t1,
-	NID_brainpoolP320r1,
-	NID_brainpoolP320t1,
-	NID_brainpoolP384r1,
-	NID_brainpoolP384t1,
-	NID_brainpoolP512r1,
-	NID_brainpoolP512t1
-#endif
-};
-
-static void init_curves()
-{
-#ifndef OPENSSL_NO_EC
-	pki_evp::num_curves = EC_get_builtin_curves(NULL, 0);
-	pki_evp::curves = (EC_builtin_curve*)OPENSSL_malloc(
-			(int)(sizeof(EC_builtin_curve) *pki_evp::num_curves));
-	if (!pki_evp::curves)
-		return;
-	EC_get_builtin_curves(pki_evp::curves, pki_evp::num_curves);
-	pki_evp::curve_flags = (unsigned char *)OPENSSL_malloc(pki_evp::num_curves);
-	if (!pki_evp::curve_flags)
-		return;
-	for (size_t i=0; i< pki_evp::num_curves; i++) {
-		size_t j;
-
-		pki_evp::curve_flags[i] = 0;
-		for (j=0; j<ARRAY_SIZE(x962_curve_nids); j++) {
-			if (x962_curve_nids[j] == pki_evp::curves[i].nid) {
-				pki_evp::curve_flags[i] = CURVE_X962;
-				break;
-			}
-		}
-		if (pki_evp::curve_flags[i])
-			continue;
-		for (j=0; j<ARRAY_SIZE(other_curve_nids); j++) {
-			if (other_curve_nids[j] == pki_evp::curves[i].nid) {
-				pki_evp::curve_flags[i] = CURVE_OTHER;
-				break;
-			}
-		}
-	}
-#endif
-}
+OidResolver *MainWindow::resolver = NULL;
 
 void MainWindow::enableTokenMenu(bool enable)
 {
@@ -197,41 +80,32 @@ void MainWindow::load_engine()
 	enableTokenMenu(pkcs11::loaded());
 }
 
-static QByteArray fileNameEncoderFunc(const QString &fileName)
-{
-	return filename2bytearray(fileName);
-}
-
-static QString fileNAmeDecoderFunc(const QByteArray &localFileName)
-{
-	return filename2QString(localFileName.constData());
-}
-
-MainWindow::MainWindow(QWidget *parent )
+MainWindow::MainWindow(QWidget *parent)
 	:QMainWindow(parent)
 {
-	QFile::setEncodingFunction(fileNameEncoderFunc);
-	QFile::setDecodingFunction(fileNAmeDecoderFunc);
-
 	dbindex = new QLabel();
 	dbindex->setFrameStyle(QFrame::Plain | QFrame::NoFrame);
 	dbindex->setMargin(6);
 
+	dn_translations_setup();
 	statusBar()->addWidget(dbindex, 1);
 
 	setupUi(this);
-	setWindowTitle(tr(XCA_TITLE));
+	setWindowTitle(XCA_TITLE);
+
+	resolver = new OidResolver(NULL);
+	resolver->setWindowTitle(XCA_TITLE);
 
 	wdList << keyButtons << reqButtons << certButtons <<
 		tempButtons <<	crlButtons;
+
+	historyMenu = NULL;
 	init_menu();
 	setItemEnabled(false);
 
 	init_images();
 	homedir = getHomeDir();
 
-	init_curves();
-
 #ifdef MDEBUG
 	CRYPTO_malloc_debug_init();
 	CRYPTO_mem_ctrl(CRYPTO_MEM_CHECK_ON);
@@ -241,15 +115,28 @@ MainWindow::MainWindow(QWidget *parent )
 	ERR_load_crypto_strings();
 	OpenSSL_add_all_algorithms();
 
+	EVP_add_digest_alias(SN_sha1,SN_ecdsa_with_SHA1);
+	EVP_add_digest_alias(SN_sha224,SN_ecdsa_with_SHA224);
+	EVP_add_digest_alias(SN_sha256,SN_ecdsa_with_SHA256);
+	EVP_add_digest_alias(SN_sha256,SN_dsa_with_SHA256);
+	EVP_add_digest_alias(SN_sha384,SN_ecdsa_with_SHA384);
+	EVP_add_digest_alias(SN_sha512,SN_ecdsa_with_SHA512);
 	/* read in all our own OIDs */
 	initOIDs();
 
 	eku_nid = read_nidlist("eku.txt");
 	dn_nid = read_nidlist("dn.txt");
 	aia_nid = read_nidlist("aia.txt");
-	connect(this, SIGNAL(newURLs(QStringList &)),
-		this, SLOT(openURLs(QStringList &)));
+
 	setAcceptDrops(true);
+
+	searchEdit = new QLineEdit();
+
+	keyView->setMainwin(this, searchEdit);
+	reqView->setMainwin(this, searchEdit);
+	certView->setMainwin(this, searchEdit);
+	tempView->setMainwin(this, searchEdit);
+	crlView->setMainwin(this, searchEdit);
 }
 
 void MainWindow::dropEvent(QDropEvent *event)
@@ -262,22 +149,29 @@ void MainWindow::dropEvent(QDropEvent *event)
 		QString s = u.toLocalFile();
 		files << s;
 	}
-	emit newURLs(files);
+	openURLs(files);
 	event->acceptProposedAction();
 }
 
 void MainWindow::openURLs(QStringList &files)
+{
+	urlsToOpen = files;
+	QTimer::singleShot(100, this, SLOT(openURLs()));
+}
+
+void MainWindow::openURLs()
 {
 	QStringList failed;
 	QString s;
 	ImportMulti *dlgi = new ImportMulti(this);
 
-	foreach(s, files) {
+	foreach(s, urlsToOpen) {
 	        pki_multi *pki = probeAnything(s);
 		if (pki && !pki->count())
 			failed << s;
 	        dlgi->addItem(pki);
 	}
+	urlsToOpen.clear();
 	dlgi->execute(1, failed);
 	delete dlgi;
 }
@@ -293,6 +187,9 @@ void MainWindow::setItemEnabled(bool enable)
 	foreach(QWidget *w, wdList) {
 		w->setEnabled(enable);
 	}
+	foreach(QWidget *w, wdMenuList) {
+		w->setEnabled(enable);
+	}
 	foreach(QAction *a, acList) {
 		a->setEnabled(enable);
 	}
@@ -357,15 +254,15 @@ void MainWindow::init_images()
 	pki_crl::icon = loadImg("crl.png");
 }
 
-void MainWindow::read_cmdline()
+void MainWindow::read_cmdline(int argc, char *argv[])
 {
 	int cnt = 1, opt = 0, force_load = 0;
 	char *arg = NULL;
 	exitApp = 0;
 	QStringList failed;
 	ImportMulti *dlgi = new ImportMulti(this);
-	while (cnt < qApp->argc()) {
-		arg = qApp->argv()[cnt];
+	while (cnt < argc) {
+		arg = argv[cnt];
 		if (arg[0] == '-') { // option
 			opt = 1;
 			switch (arg[1]) {
@@ -435,7 +332,7 @@ void MainWindow::loadPem()
 bool MainWindow::pastePem(QString text)
 {
 	bool success = false;
-	QByteArray pemdata = text.toAscii();
+	QByteArray pemdata = text.toLatin1();
 	BIO *b = BIO_QBA_mem_buf(pemdata);
 	check_oom(b);
 	pki_multi *pem = NULL;
@@ -598,6 +495,7 @@ void MainWindow::manageToken()
 				CKO_PUBLIC_KEY));
 
 		p11.startSession(slot);
+		p11.getRandom();
 		objects = p11.objectList(atts);
 
 		for (int j=0; j< objects.count(); j++) {
@@ -665,38 +563,81 @@ MainWindow::~MainWindow()
 #endif
 }
 
+void MainWindow::closeEvent(QCloseEvent *e)
+{
+	if (resolver) {
+		delete resolver;
+	}
+	QMainWindow::closeEvent(e);
+}
+
 QString makeSalt(void)
 {
 	unsigned char rand[2];
 	char saltbuf[10];
 
-	RAND_bytes(rand, 2);
+	Entropy::get(rand, 2);
 	snprintf(saltbuf, 10, "S%02X%02X", rand[0], rand[1]);
 	return QString(saltbuf);
 }
 
-void MainWindow::changeDbPass()
+int MainWindow::checkOldGetNewPass(Passwd &pass)
 {
+	QString passHash;
+	db mydb(dbfile);
 
-	Passwd pass;
+	if (!mydb.find(setting, "pwhash")) {
+		char *cpass;
+		pass_info p(tr("Current Password"),
+			tr("Please enter the current database password"), this);
+
+		if ((cpass = (char *)mydb.load(NULL))) {
+			passHash = cpass;
+			free(cpass);
+		}
+		/* Try empty password */
+		if (pki_evp::sha512passwd(pass, passHash) != passHash) {
+			/* Not the empty password, check it */
+			if (PwDialog::execute(&p, &pass, false) != 1)
+				return 0;
+		}
+
+		if (pki_evp::sha512passwd(pass, passHash) != passHash) {
+			XCA_WARN(tr("The entered password is wrong"));
+			return 0;
+		}
+	}
 
 	pass_info p(tr("New Password"), tr("Please enter the new password "
 			"to encrypt your private keys in the database-file"),
 			this);
 
-	if (PwDialog::execute(&p, &pass, true) != 1)
+	return PwDialog::execute(&p, &pass, true) != 1 ? 0 : 1;
+}
+
+void MainWindow::changeDbPass()
+{
+	Passwd pass;
+	QString tempn = dbfile + "{recrypt}";
+
+	if (!checkOldGetNewPass(pass))
 		return;
-	QString tempn = dbfile + "{new}";
+
 	try {
 		if (!QFile::copy(dbfile, tempn))
 			throw errorEx("Could not create temporary file: " +
 				tempn);
-		QString passhash = updateDbPassword(tempn, pass);
 
+		QString passhash = updateDbPassword(tempn, pass);
 		QFile new_file(tempn);
-		db mydb(dbfile);
-		mydb.mv(new_file);
+		/* closing the database erases 'dbfile' */
+		QString dbfile_bkup = dbfile;
 		close_database();
+		db mydb(dbfile_bkup);
+		if (mydb.mv(new_file))
+			throw errorEx(QString("Failed to rename %1 to %2").
+						arg(tempn).arg(dbfile_bkup));
+		dbfile = dbfile_bkup;
 		pki_evp::passHash = passhash;
 		pki_evp::passwd = pass;
 		init_database();
@@ -724,20 +665,17 @@ QString MainWindow::updateDbPassword(QString newdb, Passwd pass)
 		db_header_t head;
 
 		p = mydb.load(&head);
-		if (!p) {
-			printf("Load was empty !\n");
-			goto next;
-		}
+		if (!p)
+			throw errorEx("Failed to load item");
 		key = new pki_evp();
 		if (key->getVersion() < head.version) {
-			printf("Item[%s]: Version %d "
-				"> known version: %d -> ignored\n",
-				head.name, head.version,
-				key->getVersion()
-			);
+			int v = key->getVersion();
 			free(p);
 			delete key;
-			goto next;
+			throw errorEx(QString("Item[%1]: Version %2 "
+				"> known version: %3 -> ignored")
+				.arg(head.name).arg(head.version).arg(v)
+			);
 		}
 		key->setIntName(QString::fromUtf8(head.name));
 
@@ -746,9 +684,9 @@ QString MainWindow::updateDbPassword(QString newdb, Passwd pass)
 		}
 		catch (errorEx &err) {
 			err.appendString(key->getIntName());
-			Error(err);
+			free(p);
 			delete key;
-			key = NULL;
+			throw err;
 		}
 		free(p);
 		if (key && key->getOwnPass() == pki_key::ptCommon &&
@@ -760,7 +698,7 @@ QString MainWindow::updateDbPassword(QString newdb, Passwd pass)
 			klist << key;
 		} else if (key)
 			delete key;
-next:
+
 		if (mydb.next())
 			break;
 	}
@@ -902,8 +840,9 @@ pki_multi *MainWindow::probeAnything(QString file, int *ret)
 		if (file.endsWith(".xdb")) {
 			try {
 				int r;
-				db mydb(file);
-				mydb.verify_magic();
+				db *mydb = new db(file);
+				mydb->verify_magic();
+				delete mydb;
 				r = changeDB(file);
 				delete pki;
 				if (ret)
@@ -930,6 +869,13 @@ void MainWindow::generateDHparam()
 
 	if (!ok)
 		return;
+	/*
+	 * 1024:   6 sec
+	 * 2048:  38 sec
+	 * 4096: 864 sec
+	 */
+
+	Entropy::seed_rng();
 	try {
 		QStatusBar *status = statusBar();
 		bar = new QProgressBar();
@@ -946,7 +892,7 @@ void MainWindow::generateDHparam()
 			fname, "All files ( * )", NULL);
 		if (fname == "")
 			throw errorEx("");
-		fp = fopen(QString2filename(fname), "w");
+		fp = fopen_write(fname);
 		if (fp == NULL) {
 			throw errorEx(tr("Error opening file: '%1': %2").
 				arg(fname).arg(strerror(errno)));
@@ -964,3 +910,54 @@ void MainWindow::generateDHparam()
 		delete bar;
 }
 
+void MainWindow::changeEvent(QEvent *event)
+{
+	if (event->type() == QEvent::LanguageChange) {
+		QList<db_base*> models;
+		retranslateUi(this);
+		dn_translations_setup();
+		init_menu();
+		update_history_menu();
+		models << keys << reqs << certs << crls << temps;
+		foreach(db_base *model, models) {
+			if (model)
+				model->updateHeaders();
+		}
+		if (!dbfile.isEmpty())
+			dbindex->setText(tr("Database") + ": " + dbfile);
+	}
+	QMainWindow::changeEvent(event);
+}
+
+void MainWindow::keyPressEvent(QKeyEvent *e)
+{
+	if (e->modifiers() != Qt::ControlModifier) {
+		QMainWindow::keyPressEvent(e);
+		return;
+	}
+	int siz = XCA_application::tableFont.pointSize();
+	QList<XcaTreeView*> views;
+
+	switch (e->key()) {
+	case Qt::Key_Plus:
+		XCA_application::tableFont.setPointSize(siz +1);
+		break;
+	case Qt::Key_Minus:
+		if (siz > 4) {
+			XCA_application::tableFont.setPointSize(siz -1);
+		}
+		break;
+	default:
+		QMainWindow::keyPressEvent(e);
+		return;
+	}
+	views << keyView << reqView << certView << crlView << tempView;
+	foreach(XcaTreeView *v, views) {
+		if (v) {
+			v->header()->resizeSections(
+					QHeaderView::ResizeToContents);
+			v->reset();
+		}
+	}
+	update();
+}
