@@ -7,15 +7,15 @@
 
 
 #include "NewX509.h"
-#include <QtGui/QCheckBox>
-#include <QtGui/QComboBox>
-#include <QtGui/QRadioButton>
-#include <QtGui/QMessageBox>
-#include <QtGui/QLineEdit>
-#include <QtGui/QLabel>
-#include <QtGui/QPixmap>
-#include <QtGui/QPushButton>
-#include <QtGui/QValidator>
+#include <QCheckBox>
+#include <QComboBox>
+#include <QRadioButton>
+#include <QMessageBox>
+#include <QLineEdit>
+#include <QLabel>
+#include <QPixmap>
+#include <QPushButton>
+#include <QValidator>
 #include "MainWindow.h"
 #include "v3ext.h"
 #include "lib/x509name.h"
@@ -26,16 +26,6 @@
 #include "lib/oid.h"
 #include "lib/func.h"
 
-int NewX509::name_nid[] = {
-	NID_countryName,
-	NID_stateOrProvinceName,
-	NID_localityName,
-	NID_organizationName,
-	NID_organizationalUnitName,
-	NID_commonName,
-	NID_pkcs9_emailAddress
-};
-
 NewX509::NewX509(QWidget *parent)
 	:QDialog(parent)
 {
@@ -44,6 +34,8 @@ NewX509::NewX509(QWidget *parent)
 	dn_nid = *MainWindow::dn_nid;
 	aia_nid = *MainWindow::aia_nid;
 	attr_nid << NID_pkcs9_unstructuredName << NID_pkcs9_challengePassword;
+	foreach(QString dn, MainWindow::explicit_dn.split(","))
+		expl_dn_nid << OBJ_sn2nid(CCHAR(dn));
 
 	QStringList keys;
 
@@ -52,8 +44,8 @@ NewX509::NewX509(QWidget *parent)
 	/* temporary storage for creating temporary X509V3_CTX */
 	ctx_cert = NULL;
 
-	for (i=0; i < dn_nid.count(); i++)
-		keys << QString(OBJ_nid2ln(dn_nid[i]));
+	foreach(int nid, dn_nid)
+		keys << QString(OBJ_nid2ln(nid));
 
 	extDNlist->setKeys(keys);
 	extDNlist->setInfoLabel(extDNinfo);
@@ -120,30 +112,68 @@ NewX509::NewX509(QWidget *parent)
 	tempList->insertItems(0, strings);
 
 	// setup Extended keyusage
-	for (i=0; i < eku_nid.count(); i++)
-		ekeyUsage->addItem(OBJ_nid2ln(eku_nid[i]));
+	foreach(int nid, eku_nid)
+		ekeyUsage->addItem(OBJ_nid2ln(nid));
 
 	// setup Authority Info Access
-	for (i=0; i < aia_nid.count(); i++)
-		aiaOid->addItem(OBJ_nid2ln(aia_nid[i]));
+	foreach(int nid, aia_nid)
+		aiaOid->addItem(OBJ_nid2ln(nid));
 
 	// init the X509 v3 context
 	X509V3_set_ctx(&ext_ctx, NULL , NULL, NULL, NULL, 0);
 	X509V3_set_ctx_nodb(&ext_ctx);
 
-	QList<QLabel *> nameLabel;
-	nameLabel << LcountryName << LstateOrProvinceName << LlocalityName <<
-	LorganisationName << LorganisationalUnitName << LcommonName <<
-	LemailAddress;
+	// Setup dnWidget
+	if (dnWidget->layout())
+		delete dnWidget->layout();
+	QGridLayout *dnLayout = new QGridLayout(dnWidget);
+	dnLayout->setAlignment(Qt::AlignTop);
+	dnLayout->setSpacing(6);
+	dnLayout->setMargin(0);
+	int n = 1, col = 0;
 
-	for(int i=0; i<nameLabel.count(); i++) {
-		nameLabel[i]->setText(OBJ_nid2ln(name_nid[i]));
-		QString tt = nameLabel[i]->toolTip();
-		nameLabel[i]->setToolTip(QString("[%1] %2").
-			arg(OBJ_nid2sn(name_nid[i])).arg(tt));
-		name_ptr[i] = (QLineEdit *)nameLabel[i]->buddy();
-		setupLineEditByNid(name_nid[i], name_ptr[i]);
+	description = new QLineEdit(this);
+	description->setToolTip(tr("This name is only used internally and does not appear in the resulting certificate"));
+	QLabel *label = new QLabel(this);
+	label->setText(tr("Internal name"));
+	dnLayout->addWidget(label, 0, 0);
+	dnLayout->addWidget(description, 0, 1);
+
+	QWidget::setTabOrder(description, extDNlist);
+	QWidget *old = description;
+	foreach(int nid, expl_dn_nid) {
+		DoubleClickLabel *label;
+		QLineEdit *edit;
+		QString trans = dn_translations[nid];
+
+		label = new DoubleClickLabel(this);
+		if (translate_dn && !trans.isEmpty()) {
+			label->setText(trans);
+			label->setToolTip(QString("[%1] %2")
+				.arg(OBJ_nid2sn(nid)).arg(OBJ_nid2ln(nid)));
+		} else {
+			label->setText(OBJ_nid2ln(nid));
+			label->setToolTip(QString("[%1] %2")
+				.arg(OBJ_nid2sn(nid)).arg(trans));
+		}
+		label->setClickText(OBJ_nid2sn(nid));
+		connect(label, SIGNAL(doubleClicked(QString)),
+                        MainWindow::getResolver(), SLOT(searchOid(QString)));
+		edit = new QLineEdit(this);
+		setupLineEditByNid(nid, edit);
+		nameEdits << nameEdit(nid, edit, label);
+
+		dnLayout->addWidget(label, n, col);
+		dnLayout->addWidget(edit, n, col +1);
+		n++;
+		if (n > expl_dn_nid.size()/2 && col == 0) {
+			col = 2;
+			n = expl_dn_nid.size() & 1 ? 0 : 1;
+		}
+		QWidget::setTabOrder(old, edit);
+		old = edit;
 	}
+
 	// Setup Request Attributes
 	if (attrWidget->layout())
 		delete attrWidget->layout();
@@ -151,19 +181,34 @@ NewX509::NewX509(QWidget *parent)
 	attrLayout->setAlignment(Qt::AlignTop);
 	attrLayout->setSpacing(6);
 	attrLayout->setMargin(0);
-	attr_edit.clear();
-	for (i=0; i < attr_nid.count(); i++) {
-		QLabel *label;
+	old = reqSubChange;
+	n = 0;
+	foreach(int nid, attr_nid) {
+		DoubleClickLabel *label;
 		QLineEdit *edit;
-		int nid = attr_nid[i];
-		label = new QLabel(this);
-		label->setText(QString(OBJ_nid2ln(nid)));
-		label->setToolTip(QString(OBJ_nid2sn(nid)));
+		QString trans = dn_translations[nid];
+
+		label = new DoubleClickLabel(this);
+		if (translate_dn && !trans.isEmpty()) {
+			label->setText(trans);
+			label->setToolTip(QString(OBJ_nid2sn(nid)));
+		} else {
+			label->setText(QString(OBJ_nid2ln(nid)));
+			label->setToolTip(trans);
+		}
+		label->setClickText(OBJ_nid2sn(nid));
+		connect(label, SIGNAL(doubleClicked(QString)),
+                        MainWindow::getResolver(), SLOT(searchOid(QString)));
 		edit = new QLineEdit(this);
-		attr_edit << edit;
-		attrLayout->addWidget(label, i, 0);
-		attrLayout->addWidget(edit, i, 1);
+		attrEdits << nameEdit(nid, edit, label);
 		setupLineEditByNid(nid, edit);
+
+		attrLayout->addWidget(label, n, 0);
+		attrLayout->addWidget(edit, n, 1);
+
+		QWidget::setTabOrder(old, edit);
+		old = edit;
+		n++;
 	}
 	// last polish
 	on_certList_currentIndexChanged(0);
@@ -172,6 +217,65 @@ NewX509::NewX509(QWidget *parent)
 	attrWidget->hide();
 	pt = none;
 	notAfter->setEndDate(true);
+
+	QMap<int, DoubleClickLabel*> nidLabel;
+	nidLabel[NID_subject_alt_name] = sanLbl;
+	nidLabel[NID_issuer_alt_name] = ianLbl;
+	nidLabel[NID_crl_distribution_points] = crldpLbl;
+	nidLabel[NID_info_access] = aiaLbl;
+	nidLabel[NID_netscape_base_url] = nsBaseLbl;
+	nidLabel[NID_netscape_revocation_url] = nsRevLbl;
+	nidLabel[NID_netscape_ca_revocation_url] = nsCaRevLbl;
+	nidLabel[NID_netscape_renewal_url] = nsRenewLbl;
+	nidLabel[NID_netscape_ca_policy_url] = nsCaPolicyLbl;
+	nidLabel[NID_netscape_ssl_server_name] = nsSslServerLbl;
+	nidLabel[NID_netscape_comment] = nsCommentLbl;
+
+	foreach(int nid, nidLabel.keys()) {
+		DoubleClickLabel *l = nidLabel[nid];
+		l->setText(translate_dn ?
+			dn_translations[nid] : OBJ_nid2ln(nid));
+		if (l->toolTip().isEmpty()) {
+			l->setToolTip(translate_dn ?
+				OBJ_nid2ln(nid) : dn_translations[nid]);
+		}
+		l->setClickText(OBJ_nid2sn(nid));
+		connect(l, SIGNAL(doubleClicked(QString)),
+                        MainWindow::getResolver(), SLOT(searchOid(QString)));
+	}
+
+	QMap<int, QGroupBox*> nidGroupBox;
+	nidGroupBox[NID_basic_constraints] = bcBox;
+	nidGroupBox[NID_key_usage] = kuBox;
+	nidGroupBox[NID_ext_key_usage] = ekuBox;
+	nidGroupBox[NID_netscape_cert_type] = nsCertTypeBox;
+
+	foreach(int nid, nidGroupBox.keys()) {
+		QGroupBox *g = nidGroupBox[nid];
+		g->setTitle(translate_dn ?
+			dn_translations[nid] : OBJ_nid2ln(nid));
+		if (g->toolTip().isEmpty()) {
+			g->setToolTip(translate_dn ?
+				OBJ_nid2ln(nid) : dn_translations[nid]);
+		}
+	}
+
+	if (translate_dn) {
+		QList<QGroupBox*> gb;
+		gb << distNameBox << keyIdentBox;
+		foreach(QGroupBox *g, gb) {
+			QString tt = g->toolTip();
+			g->setToolTip(g->title());
+			g->setTitle(tt);
+		}
+		QList<QCheckBox*> cbList;
+		cbList << bcCritical << kuCritical << ekuCritical;
+		foreach(QCheckBox* cb, cbList) {
+			cb->setText(tr("Critical"));
+		}
+	}
+	if (pki_x509::disable_netscape)
+		tabWidget->removeTab(4);
 }
 
 void NewX509::setRequest()
@@ -223,22 +327,30 @@ void NewX509::setupLineEditByNid(int nid, QLineEdit *l)
 	l->setValidator(validator);
 }
 
-void NewX509::addReqAttributes(pki_x509req *req)
+void NewX509::getReqAttributes(pki_x509req *req)
 {
-	for (int i=0; i < attr_nid.count(); i++) {
-		req->addAttribute(attr_nid[i], attr_edit[i]->text());
+	foreach(nameEdit e, attrEdits) {
+		req->addAttribute(e.nid, e.edit->text());
 	}
 }
 
+void NewX509::setReqAttributes(pki_x509req *req)
+{
+	foreach(nameEdit e, attrEdits) {
+		e.edit->setText(req->getAttribute(e.nid));
+	}
+}
+
+/* Initialize dialog for Template creation */
 void NewX509::setTemp(pki_temp *temp)
 {
-	QString text = tr("Create");
+	QString text = tr("Create XCA template");
 	if (temp->getIntName() != "--") {
 		description->setText(temp->getIntName());
 		description->setDisabled(true);
-		text = tr("Edit");
+		text = tr("Edit XCA template");
 	}
-	capt->setText(text + " " + tr("XCA template"));
+	capt->setText(text);
 	tabWidget->removeTab(0);
 	privKeyBox->setEnabled(false);
 	validityBox->setEnabled(false);
@@ -246,6 +358,7 @@ void NewX509::setTemp(pki_temp *temp)
 	pt = tmpl;
 }
 
+/* Initialize dialog for Certificate creation */
 void NewX509::setCert()
 {
 	capt->setText(tr("Create x509 Certificate"));
@@ -258,12 +371,14 @@ void NewX509::setImage(QPixmap *img)
 	image->setPixmap(*img);
 }
 
+/* Select a template and apply it */
 void NewX509::defineTemplate(pki_temp *temp)
 {
 	fromTemplate(temp);
 	templateChanged(temp);
 }
 
+/* Select a Request for signing it */
 void NewX509::defineRequest(pki_x509req *req)
 {
 	if (!req)
@@ -275,6 +390,63 @@ void NewX509::defineRequest(pki_x509req *req)
 	on_fromReqCB_clicked();
 }
 
+/* Preset all values from another request to create a aimilar one */
+void NewX509::fromX509super(pki_x509super *cert_or_req)
+{
+	pki_temp *temp = new pki_temp("");
+	temp->fromCert(cert_or_req);
+	defineTemplate(temp);
+	delete temp;
+
+	description->setText(cert_or_req->getIntName());
+	pki_key *key = cert_or_req->getRefKey();
+	if (key) {
+		usedKeysToo->setChecked(true);
+		keyList->setCurrentIndex(private_keys.indexOf(
+			key->getIntNameWithType()));
+	}
+	hashAlgo->setCurrentMD(cert_or_req->getDigest());
+
+	switch(cert_or_req->getType()) {
+	case x509: {
+		pki_x509 *cert = (pki_x509*)cert_or_req;
+		pki_x509 *signer = cert->getSigner();
+		if (signer == cert) {
+			foreignSignRB->setChecked(false);
+		} else if (signer) {
+			defineSigner(signer);
+		}
+		notBefore->setDate(cert->getNotBefore());
+		notAfter->setDate(cert->getNotAfter());
+		break;
+	}
+	case x509_req: {
+		pki_x509req *req = (pki_x509req*)cert_or_req;
+		setReqAttributes(req);
+		break;
+	}
+	default:
+		break;
+	}
+
+}
+
+/* Preset all values from another cert to create a aimilar one */
+void NewX509::defineCert(pki_x509 *cert)
+{
+	fromX509super(cert);
+
+	pki_x509 *signer = cert->getSigner();
+	if (signer == cert) {
+		foreignSignRB->setChecked(false);
+	} else if (signer) {
+		defineSigner(signer);
+	}
+	notBefore->setDate(cert->getNotBefore());
+	notAfter->setDate(cert->getNotAfter());
+}
+
+/* Preset the signing certificate */
 void NewX509::defineSigner(pki_x509 *defcert)
 {
 	int index;
@@ -501,9 +673,9 @@ void NewX509::on_genKeyBut_clicked()
 {
 	QString name = description->text();
 	if (name.isEmpty())
-		name = commonName->text();
+		name = dnEntryByNid(NID_commonName);
 	if (name.isEmpty())
-		name = emailAddress->text();
+		name = dnEntryByNid(NID_pkcs9_emailAddress);
 	emit genKey(name);
 }
 
@@ -633,9 +805,8 @@ x509name NewX509::getX509name(int _throw)
 	int j, row, nid;
 
 	try {
-		for (j = 0; j<EXPLICIT_NAME_CNT; j++) {
-			nid = name_nid[j];
-			x.addEntryByNid(nid, name_ptr[j]->text());
+		foreach(nameEdit ne, nameEdits) {
+			x.addEntryByNid(ne.nid, ne.edit->text());
 		}
 		row = extDNlist->rowCount();
 		for (j=0; j<row; j++) {
@@ -654,24 +825,38 @@ x509name NewX509::getX509name(int _throw)
 	return x;
 }
 
+QString NewX509::dnEntryByNid(int nid)
+{
+	foreach(nameEdit ne, nameEdits) {
+		if (ne.nid == nid && !ne.edit->text().isEmpty())
+			return ne.edit->text();
+	}
+	for (int j=0; j<extDNlist->rowCount(); j++) {
+		QStringList l = extDNlist->getRow(j);
+	        if (OBJ_txt2nid(CCHAR(l[0])) == nid && !l[1].isEmpty())
+			return l[1];
+	}
+	return QString();
+}
+
 void NewX509::setX509name(const x509name &n)
 {
-	int i,j;
-
 	extDNlist->deleteAllRows();
-	for (j=0; j<EXPLICIT_NAME_CNT; j++) {
-		name_ptr[j]->setText("");
+	foreach(nameEdit ne, nameEdits) {
+		ne.edit->setText("");
 	}
-	for (i=0, j=0; i< n.entryCount(); i++) {
+	for (int i=0; i< n.entryCount(); i++) {
 		int nid = n.nid(i);
+		bool done = false;
 		QStringList sl = n.entryList(i);
-		for ( ; j<EXPLICIT_NAME_CNT; j++) {
-			if (nid == name_nid[j] && name_ptr[j]->text().isEmpty()) {
-				name_ptr[j]->setText(sl[2]);
+		foreach(nameEdit ne, nameEdits) {
+			if (nid == ne.nid && ne.edit->text().isEmpty()) {
+				ne.edit->setText(sl[2]);
+				done = true;
 				break;
 			}
 		}
-		if (j == EXPLICIT_NAME_CNT) {
+		if (!done) {
 			extDNlist->addRow(sl.mid(1, 2));
 		}
 	}
@@ -830,7 +1015,8 @@ int NewX509::validateExtensions(QString nconf, QString &result)
 	(void)nconf;
 	try {
 		el = getGuiExt();
-		el += getNetscapeExt();
+		if (!pki_x509::disable_netscape)
+			el += getNetscapeExt();
 		el.delInvalid();
 	} catch (errorEx &err) {
 		errors += err.getString();
@@ -999,7 +1185,8 @@ void NewX509::accept()
 		}
 	}
 	if (description->text().isEmpty() && !fromReqCB->isChecked()) {
-		if (commonName->text().isEmpty()) {
+		QString cn = dnEntryByNid(NID_commonName);
+		if (cn.isEmpty()) {
 			gotoTab(1);
 			xcaWarning msg(this,
 				tr("The internal name and the common name are empty.\nPlease set at least the internal name."));
@@ -1010,7 +1197,7 @@ void NewX509::accept()
 			}
 			return;
 		} else {
-			description->setText(commonName->text());
+			description->setText(cn);
 		}
 	}
 	if (keyList->count() == 0 && keyList->isEnabled() &&
@@ -1119,13 +1306,14 @@ void NewX509::accept()
 				notAfter->setDate(signer->getNotAfter());
 		}
 	}
-	if (notBefore->getDate() > notAfter->getDate()) {
+	if (validityBox->isEnabled() &&
+	    notBefore->getDate() > notAfter->getDate()) {
 		gotoTab(2);
 		QString text = tr("The certificate will be out of date before it becomes valid. You most probably mixed up both dates.");
 		xcaWarning msg(this, text);
 		msg.addButton(QMessageBox::Ok)->setText(tr("Edit dates"));
 		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
-		msg.addButton(QMessageBox::Apply)->setText("Continue rollout");
+		msg.addButton(QMessageBox::Apply)->setText(tr("Continue rollout"));
 		switch (msg.exec())
 		{
 			case QMessageBox::Ok:
