@@ -14,12 +14,13 @@
 #include "lib/db.h"
 #include "widgets/kvView.h"
 #include <openssl/x509v3.h>
-#include <QtGui/QListWidget>
+#include <QListWidget>
 
 class MainWindow;
 class extList;
 class pki_temp;
 class pki_x509req;
+class pki_x509super;
 class pki_x509;
 class pki_key;
 class QPixmap;
@@ -28,6 +29,16 @@ class x509name;
 class x509v3ext;
 class extList;
 
+class nameEdit {
+    public:
+	int nid;
+	QLineEdit *edit;
+	QLabel *label;
+	nameEdit(int n, QLineEdit *e, QLabel *l) {
+		nid = n; edit = e; label = l;
+	}
+};
+
 class NewX509: public QDialog, public Ui::NewX509
 {
 		Q_OBJECT
@@ -36,9 +47,9 @@ class NewX509: public QDialog, public Ui::NewX509
 		NIDlist dn_nid;
 		NIDlist aia_nid;
 		NIDlist attr_nid;
-		QList<QLineEdit*> attr_edit;
-		#define EXPLICIT_NAME_CNT 7
-		QLineEdit *name_ptr[EXPLICIT_NAME_CNT];
+		NIDlist expl_dn_nid;
+		QList<nameEdit> attrEdits;
+		QList<nameEdit> nameEdits;
 		X509V3_CTX ext_ctx;
 		void editV3ext(QLineEdit *le, QString types, int n);
 		enum pki_type pt;
@@ -50,10 +61,11 @@ class NewX509: public QDialog, public Ui::NewX509
 		kvmodel *extDNmodel;
 		extList getExtDuplicates();
 		void checkIcon(const QString &text, int nid, QLabel*img);
+		QString dnEntryByNid(int nid);
 
 	public:
-		static int name_nid[EXPLICIT_NAME_CNT];
 		QRadioButton *selfQASignRB;
+		QLineEdit *description;
 		NewX509(QWidget *parent);
 		virtual ~NewX509();
 		void initCtx();
@@ -63,8 +75,10 @@ class NewX509: public QDialog, public Ui::NewX509
 		void toTemplate(pki_temp *temp);
 		void fromTemplate(pki_temp *temp);
 		void defineTemplate(pki_temp *temp);
+		void defineCert(pki_x509 *cert);
 		void defineRequest(pki_x509req *req);
 		void defineSigner(pki_x509 *defcert);
+		void fromX509super(pki_x509super *cert_or_req);
 		void templateChanged(pki_temp *templ);
 		pki_key *getSelectedKey();
 		pki_x509 *getSelectedSigner();
@@ -89,9 +103,9 @@ class NewX509: public QDialog, public Ui::NewX509
 		extList getAllExt();
 		void setupTmpCtx();
 		void initCtx(pki_x509 *subj, pki_x509 *iss, pki_x509req *req);
-		void setExt(const x509v3ext &ext);
 		void switchHashAlgo();
-		void addReqAttributes(pki_x509req *req);
+		void setReqAttributes(pki_x509req *req);
+		void getReqAttributes(pki_x509req *req);
 		int checkExtDuplicates();
 		void subjectFromTemplate(pki_temp *temp);
 		void extensionsFromTemplate(pki_temp *temp);
