@@ -7,12 +7,12 @@
 
 
 #include "NewX509.h"
-#include <QtGui/QCheckBox>
-#include <QtGui/QComboBox>
-#include <QtGui/QRadioButton>
-#include <QtGui/QLineEdit>
-#include <QtGui/QListWidget>
-#include <QtGui/QMessageBox>
+#include <QCheckBox>
+#include <QComboBox>
+#include <QRadioButton>
+#include <QLineEdit>
+#include <QListWidget>
+#include <QMessageBox>
 
 #include "MainWindow.h"
 #include "lib/x509v3ext.h"
@@ -209,7 +209,7 @@ extList NewX509::getAdvanced()
 	if (conf_str.isEmpty())
 		return elist;
 
-	QByteArray cs = conf_str.toAscii();
+	QByteArray cs = conf_str.toLatin1();
 	bio = BIO_new_mem_buf(cs.data(), cs.length());
 	if (!bio)
 		return elist;
@@ -267,7 +267,8 @@ extList NewX509::getAllExt()
 	extList ne;
 	ne = getGuiExt();
 	ne += getAdvanced();
-	ne += getNetscapeExt();
+	if (!pki_x509::disable_netscape)
+		ne += getNetscapeExt();
 	return ne;
 }
 
@@ -290,13 +291,13 @@ extList NewX509::getNetscapeExt()
 	}
 
 	el << ext.create(NID_netscape_cert_type, cont.join(", "), &ext_ctx);
-	el << ext.create(NID_netscape_base_url, nsBaseUrl->text(), &ext_ctx);
-	el << ext.create(NID_netscape_revocation_url, nsRevocationUrl->text(), &ext_ctx);
-	el << ext.create(NID_netscape_ca_revocation_url, nsCARevocationUrl->text(), &ext_ctx);
-	el << ext.create(NID_netscape_renewal_url, nsRenewalUrl->text(), &ext_ctx);
-	el << ext.create(NID_netscape_ca_policy_url, nsCaPolicyUrl->text(), &ext_ctx);
-	el << ext.create(NID_netscape_ssl_server_name, nsSslServerName->text(), &ext_ctx);
-	el << ext.create(NID_netscape_comment, nsComment->text(), &ext_ctx);
+	el << ext.create_ia5(NID_netscape_base_url, nsBaseUrl->text(), &ext_ctx);
+	el << ext.create_ia5(NID_netscape_revocation_url, nsRevocationUrl->text(), &ext_ctx);
+	el << ext.create_ia5(NID_netscape_ca_revocation_url, nsCARevocationUrl->text(), &ext_ctx);
+	el << ext.create_ia5(NID_netscape_renewal_url, nsRenewalUrl->text(), &ext_ctx);
+	el << ext.create_ia5(NID_netscape_ca_policy_url, nsCaPolicyUrl->text(), &ext_ctx);
+	el << ext.create_ia5(NID_netscape_ssl_server_name, nsSslServerName->text(), &ext_ctx);
+	el << ext.create_ia5(NID_netscape_comment, nsComment->text(), &ext_ctx);
 	return el;
 }
 
@@ -342,12 +343,3 @@ extList NewX509::getExtDuplicates()
         }
 	return el_dup;
 }
-
-void NewX509::setExt(const x509v3ext &ext)
-{
-	switch (ext.nid()) {
-		case NID_basic_constraints:
-			bcCritical->setChecked(ext.getCritical());
-	}
-}
-
