@@ -9,7 +9,7 @@
 #define __OPTIONS_H
 
 #include "ui_Options.h"
-#include <QtGui/QDialog>
+#include <QDialog>
 #include "lib/base.h"
 #include "SearchPkcs11.h"
 #include "MainWindow.h"
@@ -20,20 +20,35 @@ class Options: public QDialog, public Ui::Options
 	private:
 		SearchPkcs11 *searchP11;
 		QStringList string_opts;
+		QString getDnString(QListWidget *w);
+		void setDnString(QString dn, QListWidget *w);
 		MainWindow *mw;
 	public:
 		Options(MainWindow *parent);
 		~Options();
 		void setupPkcs11Provider(QString list);
 		void setStringOpt(const QString string_opt);
-		QString getDnString();
+		QString getExtDnString() {
+			return getDnString(extDNlist);
+		}
+		QString getExpDnString() {
+			return getDnString(expDNlist);
+		}
 		QString getStringOpt();
 		QString getPkcs11Provider();
-		void setDnString(QString dn);
+		void setExpDnString(QString dn) {
+			setDnString(dn, expDNlist);
+		}
+		void setExtDnString(QString dn) {
+			setDnString(dn, extDNlist);
+		}
 
 	public slots:
 		void on_extDNadd_clicked();
 		void on_extDNdel_clicked();
+		void on_expDNadd_clicked();
+		void on_expDNdel_clicked();
+		void on_expDNdefault_clicked();
 		void on_addButton_clicked(void);
 		void on_removeButton_clicked(void);
 		void on_searchPkcs11_clicked(void);
