@@ -8,7 +8,7 @@
 #ifndef __HASH_BOX_H
 #define __HASH_BOX_H
 
-#include <QtGui/QComboBox>
+#include <QComboBox>
 #include <openssl/evp.h>
 
 class hashBox: public QComboBox
@@ -24,6 +24,7 @@ class hashBox: public QComboBox
 		void setKeyType(int type);
 		const EVP_MD *currentHash();
 		QString currentHashName();
+		void setCurrentMD(const EVP_MD *md);
 		void setCurrentAsDefault();
 		void setDefaultHash();
 		void setupHashes(QList<int> nids);
